// date-time.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

int compute_julian_day(int yyyy, int mm, int dd) {
  // Returns the Julian day that starts at noon on yyyy-mm-dd.
  // `yyyy' is the year.  `mm' is the month.  `dd' is the day of month.
  // Negative years are B.C.  Remember there is no year zero.
  { int jul = 0;
    int ja = 0;
    int jy = 0;
    int jm = 0;

    if (yyyy == 0) {
      throw *new_stella_exception("There is no year zero.");
    }
    if (mm > 12) {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "Month " << "`" << mm << "'" << " must be <= 12";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
    if (mm < 0) {
      { Output_String_Stream* stream_001 = new_output_string_stream();

        *(stream_001->native_stream) << "Month " << "`" << mm << "'" << " must be > zero.";
        throw *new_stella_exception(stream_001->the_string_reader());
      }
    }
    if (dd > 31) {
      { Output_String_Stream* stream_002 = new_output_string_stream();

        *(stream_002->native_stream) << "Day " << "`" << dd << "'" << " must be <= 31";
        throw *new_stella_exception(stream_002->the_string_reader());
      }
    }
    if (dd < 0) {
      { Output_String_Stream* stream_003 = new_output_string_stream();

        *(stream_003->native_stream) << "Day " << "`" << dd << "'" << " must be > zero.";
        throw *new_stella_exception(stream_003->the_string_reader());
      }
    }
    if (yyyy < 0) {
      yyyy = yyyy + 1;
    }
    if (mm > 2) {
      {
        jy = yyyy;
        jm = mm + 1;
      }
    }
    else {
      {
        jy = yyyy - 1;
        jm = mm + 13;
      }
    }
    jul = floor_(365.25 * jy) + floor_(30.6001 * jm) + dd + 1720995;
    if ((dd + (31 * (mm + (12 * yyyy)))) >= (15 + (31 * (10 + (12 * 1582))))) {
      ja = ((int)(0.01 * jy));
      jul = jul + (2 - ja) + ((int)(0.25 * ja));
    }
    return (jul);
  }
}

int julian_day_to_modified_julian_day(int julian_day) {
  // Returns the modified Julian day during which `julian-day'starts at noon.
  return (julian_day - 2400000);
}

int modified_julian_day_to_julian_day(int modified_julian_day) {
  // Returns the modified Julian day during which `julian-day'starts at noon.
  return (modified_julian_day + 2400000);
}

Keyword* compute_day_of_week_julian(int julian_day) {
  // Returns the day of the week for julian-day
  switch ((julian_day % 7)) {
    case 0: 
      return (KWD_DATE_TIME_MONDAY);
    case 1: 
      return (KWD_DATE_TIME_TUESDAY);
    case 2: 
      return (KWD_DATE_TIME_WEDNESDAY);
    case 3: 
      return (KWD_DATE_TIME_THURSDAY);
    case 4: 
      return (KWD_DATE_TIME_FRIDAY);
    case 5: 
      return (KWD_DATE_TIME_SATURDAY);
    case 6: 
      return (KWD_DATE_TIME_SUNDAY);
    default:
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << (julian_day % 7) << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    break;
  }
}

Keyword* compute_day_of_week(int yyyy, int mm, int dd) {
  // Returns the day of the week for yyyy-mm-dd.
  return (compute_day_of_week_julian(compute_julian_day(yyyy, mm, dd)));
}

int compute_next_moon_phase(int n, Keyword* phase, double& _return1) {
  // Returns the Julian Day and fraction of day of the Nth occurence
  // since January 1, 1900 of moon PHASE.  PHASE is one of :NEW-MOON, :FIRST-QUARTER,
  // :FULL-MOON, :LAST-QUARTER
  { int i = 0;
    double am = 0.0;
    double as = 0.0;
    double c = 0.0;
    double x = 0.0;
    double x2 = 0.0;
    double extra = 0.0;
    double rad = PI / 180.0;
    int julian_day = 0;

    if (phase == KWD_DATE_TIME_NEW_MOON) {
      c = ((double)(n));
      julian_day = 0;
    }
    else if (phase == KWD_DATE_TIME_FIRST_QUARTER) {
      c = n + 0.25;
      julian_day = 7;
    }
    else if (phase == KWD_DATE_TIME_FULL_MOON) {
      c = n + 0.5;
      julian_day = 14;
    }
    else if (phase == KWD_DATE_TIME_LAST_QUARTER) {
      c = n + 0.75;
      julian_day = 21;
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << phase << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
    x = c / 1236.85;
    x2 = x * x;
    as = 359.2242 + (29.105356 * c);
    am = 306.0253 + (385.816918 * c) + (0.01073 * x2);
    julian_day = 2415020 + (28 * n) + julian_day;
    extra = 0.75933 + (1.53058868 * c) + ((1.1779999999999999e-4 - (1.55e-7 * x)) * x2);
    if ((phase == KWD_DATE_TIME_NEW_MOON) ||
        (phase == KWD_DATE_TIME_FULL_MOON)) {
      extra = extra + (((0.1734 - (3.93e-4 * x)) * sin((as * rad))) - (0.4068 * sin((am * rad))));
    }
    else if ((phase == KWD_DATE_TIME_FIRST_QUARTER) ||
        (phase == KWD_DATE_TIME_LAST_QUARTER)) {
      extra = extra + (((0.1721 - (4.0e-4 * x)) * sin((as * rad))) - (0.628 * sin((am * rad))));
    }
    else {
      { Output_String_Stream* stream_001 = new_output_string_stream();

        *(stream_001->native_stream) << "`" << phase << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_001->the_string_reader());
      }
    }
    i = ((extra >= 0.0) ? floor_(extra) : ceiling(extra - 1.0));
    { int _return_temp = julian_day + i;

      _return1 = extra - i;
      return (_return_temp);
    }
  }
}

int compute_calendar_date(int julian_day, int& _return1, int& _return2, Keyword*& _return3) {
  // Returns the YEAR, MONTH, DAY, DAY-OF-WEEK on which the given
  // `julian-day' begins at noon.
  { int ja = julian_day;
    int jalpha = 0;
    int jb = 0;
    int jc = 0;
    int jd = 0;
    int je = 0;
    int igreg = 2299161;
    int yyyy = 0;
    int mm = 0;
    int dd = 0;

    if (julian_day >= igreg) {
      jalpha = ((int)(((julian_day - 1867216) - 0.25) / 36524.25));
      ja = julian_day + 1 + jalpha + ((int)(-0.25 * jalpha));
    }
    jb = ja + 1524;
    jc = ((int)(6680.0 + (((jb - 2439870) - 122.1) / 365.25)));
    jd = ((int)((365 * jc) + (0.25 * jc)));
    je = ((int)((jb - jd) / 30.6001));
    dd = (jb - jd) - ((int)(30.6001 * je));
    mm = je - 1;
    if (mm > 12) {
      mm = mm - 12;
    }
    yyyy = jc - 4715;
    if (mm > 2) {
      yyyy = yyyy - 1;
    }
    if (yyyy <= 0) {
      yyyy = yyyy - 1;
    }
    { int _return_temp = yyyy;

      _return1 = mm;
      _return2 = dd;
      _return3 = compute_day_of_week_julian(julian_day);
      return (_return_temp);
    }
  }
}

# include <time.h>
int get_current_date_time(int& _return1, int& _return2, Keyword*& _return3, int& _return4, int& _return5, int& _return6, int& _return7) {
  // Returns the current time in UTC as multiple values of
  // year month day day-of-week hour minute second millisecond.  Currently
  // millisecond will always be zero (even in Java where it is technically available).
  { int year = 0;
    int month = 0;
    int day = 0;
    int hour = 0;
    int minute = 0;
    int second = 0;
    int millisecond = 0;

    time_t now = time(0);
      if (now > 0) {
        struct tm* time = gmtime(&now);
        year  = time->tm_year + 1900;
        month = time->tm_mon + 1;
        day   = time->tm_mday;
        hour  = time->tm_hour;
        minute = time->tm_min;
        second = time->tm_sec;
      };
    { int _return_temp = year;

      _return1 = month;
      _return2 = day;
      _return3 = compute_day_of_week(year, month, day);
      _return4 = hour;
      _return5 = minute;
      _return6 = second;
      _return7 = millisecond;
      return (_return_temp);
    }
  }
}

double get_local_time_zone() {
  // Returns the current time zone offset from UTC as a float.
  { double offset = 0.0;

    tzset();
    offset = - timezone / (60.0 * 60.0);
    // `altzone' not available under Linux, so assume 1-hour offset:
    if (daylight > 0) offset += 1.0;;
    return (offset);
  }
}

int MILLIS_PER_DAY = NULL_INTEGER;

int MILLIS_PER_HOUR = NULL_INTEGER;

// A vector of month abbreviations for printing dates
Vector* oMONTH_ABBREVIATION_VECTORo = NULL;

Decoded_Date_Time* new_decoded_date_time() {
  { Decoded_Date_Time* self = NULL;

    self = new Decoded_Date_Time();
    self->date_time_dow = KWD_DATE_TIME_SUNDAY;
    self->date_time_zone = 0.0;
    self->date_time_year = 1900;
    self->date_time_month = 1;
    self->date_time_day = 1;
    self->date_time_hour = 0;
    self->date_time_minute = 0;
    self->date_time_second = 0;
    self->date_time_millis = 0;
    return (self);
  }
}

Surrogate* Decoded_Date_Time::primary_type() {
  { Decoded_Date_Time* self = this;

    return (SGT_DATE_TIME_STELLA_DECODED_DATE_TIME);
  }
}

Object* access_decoded_date_time_slot_value(Decoded_Date_Time* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_MILLIS) {
    if (setvalueP) {
      self->date_time_millis = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->date_time_millis);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_SECOND) {
    if (setvalueP) {
      self->date_time_second = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->date_time_second);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_MINUTE) {
    if (setvalueP) {
      self->date_time_minute = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->date_time_minute);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_HOUR) {
    if (setvalueP) {
      self->date_time_hour = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->date_time_hour);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_DAY) {
    if (setvalueP) {
      self->date_time_day = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->date_time_day);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_MONTH) {
    if (setvalueP) {
      self->date_time_month = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->date_time_month);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_YEAR) {
    if (setvalueP) {
      self->date_time_year = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->date_time_year);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_ZONE) {
    if (setvalueP) {
      self->date_time_zone = ((Float_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_float(self->date_time_zone);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_DATE_TIME_DOW) {
    if (setvalueP) {
      self->date_time_dow = ((Keyword*)(value));
    }
    else {
      value = self->date_time_dow;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void set_current_time(Decoded_Date_Time* values_structure) {
  { int year = NULL_INTEGER;
    int mon = NULL_INTEGER;
    int day = NULL_INTEGER;
    Keyword* dow = NULL;
    int hour = NULL_INTEGER;
    int min = NULL_INTEGER;
    int sec = NULL_INTEGER;
    int millis = NULL_INTEGER;

    year = get_current_date_time(mon, day, dow, hour, min, sec, millis);
    {
      year = year;
      mon = mon;
      day = day;
      dow = dow;
    }
    values_structure->date_time_millis = millis;
    values_structure->date_time_second = sec;
    values_structure->date_time_minute = min;
    values_structure->date_time_hour = hour;
  }
}

void set_current_date(Decoded_Date_Time* values_structure) {
  { int year = NULL_INTEGER;
    int mon = NULL_INTEGER;
    int day = NULL_INTEGER;
    Keyword* dow = NULL;
    int hour = NULL_INTEGER;
    int min = NULL_INTEGER;
    int sec = NULL_INTEGER;
    int millis = NULL_INTEGER;

    year = get_current_date_time(mon, day, dow, hour, min, sec, millis);
    {
      hour = hour;
      min = min;
      sec = sec;
      millis = millis;
    }
    values_structure->date_time_dow = dow;
    values_structure->date_time_day = day;
    values_structure->date_time_month = mon;
    values_structure->date_time_year = year;
  }
}

void set_current_date_time(Decoded_Date_Time* values_structure) {
  { int year = NULL_INTEGER;
    int mon = NULL_INTEGER;
    int day = NULL_INTEGER;
    Keyword* dow = NULL;
    int hour = NULL_INTEGER;
    int min = NULL_INTEGER;
    int sec = NULL_INTEGER;
    int millis = NULL_INTEGER;

    year = get_current_date_time(mon, day, dow, hour, min, sec, millis);
    values_structure->date_time_millis = millis;
    values_structure->date_time_second = sec;
    values_structure->date_time_minute = min;
    values_structure->date_time_hour = hour;
    values_structure->date_time_dow = dow;
    values_structure->date_time_day = day;
    values_structure->date_time_month = mon;
    values_structure->date_time_year = year;
  }
}

Calendar_Date* new_calendar_date() {
  { Calendar_Date* self = NULL;

    self = new Calendar_Date();
    self->time_millis = NULL_INTEGER;
    self->modified_julian_day = NULL_INTEGER;
    return (self);
  }
}

Surrogate* Calendar_Date::primary_type() {
  { Calendar_Date* self = this;

    return (SGT_DATE_TIME_STELLA_CALENDAR_DATE);
  }
}

Object* access_calendar_date_slot_value(Calendar_Date* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_DATE_TIME_STELLA_MODIFIED_JULIAN_DAY) {
    if (setvalueP) {
      self->modified_julian_day = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->modified_julian_day);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_TIME_MILLIS) {
    if (setvalueP) {
      self->time_millis = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->time_millis);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Calendar_Date::print_object(ostream* stream) {
  { Calendar_Date* self = this;

    print_calendar_date(self, stream);
  }
}

char* calendar_date_to_string(Calendar_Date* date, double timezone) {
  // Returns a string representation of `date' adjusted for `timezone'
  { int year = NULL_INTEGER;
    int month = NULL_INTEGER;
    int day = NULL_INTEGER;
    Keyword* dow = NULL;

    year = date->get_calendar_date(timezone, month, day, dow);
    dow = dow;
    { int hours = NULL_INTEGER;
      int minutes = NULL_INTEGER;
      int seconds = NULL_INTEGER;
      int milli = NULL_INTEGER;

      hours = date->get_time(timezone, minutes, seconds, milli);
      return (string_concatenate(((year < 0) ? string_concatenate(integer_to_string(0 - year), "BC", 0) : integer_to_string(year)), "-", 11, ((String_Wrapper*)((oMONTH_ABBREVIATION_VECTORo->the_array)[month]))->wrapper_value, "-", format_with_padding(integer_to_string(day), 2, '0', KWD_DATE_TIME_RIGHT, FALSE), " ", integer_to_string(hours), ":", format_with_padding(integer_to_string(minutes), 2, '0', KWD_DATE_TIME_RIGHT, FALSE), ":", format_with_padding(integer_to_string(seconds), 2, '0', KWD_DATE_TIME_RIGHT, FALSE), ".", format_with_padding(integer_to_string(milli), 3, '0', KWD_DATE_TIME_RIGHT, FALSE)));
    }
  }
}

time_t calendar_date_to_native_date_time(Calendar_Date* date) {
  { int mjd = date->modified_julian_day;
    int millis = date->time_millis;

    return (((mjd - 40588) * 86400) + (millis / 1000));
  }
}

Calendar_Date* native_date_time_to_calendar_date(time_t date) {
  { int mjd = 0;
    int millis = 0;

    mjd =  (int) ((date / 86400) + 40588);
      millis = (int) ((date % 86400) * 1000);
    return (make_calendar_date(mjd, millis));
  }
}

void print_calendar_date(Calendar_Date* date, ostream* stream) {
  if (oPRINTREADABLYpo.get()) {
    *(stream) << "\"";
  }
  else {
    *(stream) << "<" << date->modified_julian_day << " ";
  }
  *(stream) << calendar_date_to_string(date, 0.0);
  if (oPRINTREADABLYpo.get()) {
    *(stream) << " UTC\"";
  }
  else {
    *(stream) << " UTC>";
  }
}

Relative_Date* new_relative_date() {
  { Relative_Date* self = NULL;

    self = new Relative_Date();
    self->millis = NULL_INTEGER;
    self->days = NULL_INTEGER;
    return (self);
  }
}

Surrogate* Relative_Date::primary_type() {
  { Relative_Date* self = this;

    return (SGT_DATE_TIME_STELLA_RELATIVE_DATE);
  }
}

Object* access_relative_date_slot_value(Relative_Date* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_DATE_TIME_STELLA_DAYS) {
    if (setvalueP) {
      self->days = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->days);
    }
  }
  else if (slotname == SYM_DATE_TIME_STELLA_MILLIS) {
    if (setvalueP) {
      self->millis = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->millis);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Relative_Date::print_object(ostream* stream) {
  { Relative_Date* self = this;

    print_relative_date(self, stream);
  }
}

char* relative_date_to_string(Relative_Date* date) {
  // Returns a string representation of `date'
  { int n_days = date->days;
    int n_millis = date->millis;

    if (n_days < 0) {
      if (n_millis < 0) {
        return (string_concatenate("minus ", integer_to_string(0 - n_days), 3, " days; ", integer_to_string(n_millis), " ms"));
      }
      else {
        return (string_concatenate("minus ", integer_to_string(0 - (n_days + 1)), 3, " days; ", integer_to_string(MILLIS_PER_DAY - n_millis), " ms"));
      }
    }
    else {
      if (n_millis < 0) {
        if (n_days > 0) {
          return (string_concatenate("plus ", integer_to_string(n_days - 1), 3, " days; ", integer_to_string(MILLIS_PER_DAY + n_millis), " ms"));
        }
        else {
          return (string_concatenate("minus 0 days; ", integer_to_string(0 - n_millis), 1, " ms"));
        }
      }
      else {
        return (string_concatenate("plus ", integer_to_string(n_days), 3, " days; ", integer_to_string(n_millis), " ms"));
      }
    }
  }
}

void print_relative_date(Relative_Date* date, ostream* stream) {
  *(stream) << "<" << relative_date_to_string(date) << ">";
}

Calendar_Date* make_date_time(int year, int month, int day, int hour, int minute, int second, int millis, double timezone) {
  // Create a calendar date with the specified components.  `year' must be
  // the complete year (i.e., a year of 98 is 98 A.D in the 1st century).
  // `timezone' is a real number in the range -12.0 to +14.0 where UTC is zone 0.0;  The number
  // is the number of hours to add to UTC to arrive at local time.
  { int time = (hour * MILLIS_PER_HOUR) + (minute * 60000) + (second * 1000) + millis;

    time = ((int)(time - (timezone * MILLIS_PER_HOUR)));
    return (make_calendar_date(julian_day_to_modified_julian_day(compute_julian_day(year, month, day)), time));
  }
}

Calendar_Date* make_current_date_time() {
  // Create a calendar date with current time and date.
  { int year = NULL_INTEGER;
    int month = NULL_INTEGER;
    int day = NULL_INTEGER;
    Keyword* dow = NULL;
    int hour = NULL_INTEGER;
    int minute = NULL_INTEGER;
    int second = NULL_INTEGER;
    int millisecond = NULL_INTEGER;

    year = get_current_date_time(month, day, dow, hour, minute, second, millisecond);
    dow = dow;
    return (make_date_time(year, month, day, hour, minute, second, millisecond, 0.0));
  }
}

Calendar_Date* make_calendar_date(int day, int time) {
  while (time < 0) {
    time = time + MILLIS_PER_DAY;
    day = day - 1;
  }
  while (time > MILLIS_PER_DAY) {
    time = time - MILLIS_PER_DAY;
    day = day + 1;
  }
  { Calendar_Date* self_000 = new_calendar_date();

    self_000->modified_julian_day = day;
    self_000->time_millis = time;
    { Calendar_Date* value_000 = self_000;

      return (value_000);
    }
  }
}

Relative_Date* make_relative_date(int days, int millis) {
  while (millis < (0 - MILLIS_PER_DAY)) {
    millis = millis + MILLIS_PER_DAY;
    days = days - 1;
  }
  while (millis > MILLIS_PER_DAY) {
    millis = millis - MILLIS_PER_DAY;
    days = days + 1;
  }
  if (days < 0) {
    if (!(millis < 0)) {
      days = days + 1;
      millis = millis - MILLIS_PER_DAY;
    }
  }
  else if (days > 0) {
    if (!(millis >= 0)) {
      days = days - 1;
      millis = millis + MILLIS_PER_DAY;
    }
  }
  else {
  }
  { Relative_Date* self_000 = new_relative_date();

    self_000->days = days;
    self_000->millis = millis;
    { Relative_Date* value_000 = self_000;

      return (value_000);
    }
  }
}

int decode_time_in_millis(int time, int& _return1, int& _return2, int& _return3) {
  // Returns multiple values of hours, minutes, seconds, milliseconds for
  // `time' specified in milliseconds.
  { int millis = (time % 1000);
    int secs = ((floor_(time / 1000.0)) % 60);
    int mins = ((floor_(time / 60000.0)) % 60);
    int hours = floor_(((double)(time)) / MILLIS_PER_HOUR);

    _return1 = mins;
    _return2 = secs;
    _return3 = millis;
    return (hours);
  }
}

int Calendar_Date::get_calendar_date(double timezone, int& _return1, int& _return2, Keyword*& _return3) {
  // Returns multiple values of year, month, day and day of week for `date'
  // in `timezone'.  `timezone' is the number of hours added to UTC to get local time.  It
  // is in the range -12.0 to +14.0 where UTC is zone 0.0
  { Calendar_Date* date = this;

    { int time = date->time_millis;
      int timezone_adjusted_time = ((int)(time + (timezone * MILLIS_PER_HOUR)));
      int day_offset = 0;

      if (timezone_adjusted_time < 0) {
        day_offset = -1;
      }
      else if (timezone_adjusted_time > MILLIS_PER_DAY) {
        day_offset = 1;
      }
      else {
      }
      return (compute_calendar_date(modified_julian_day_to_julian_day(date->modified_julian_day + day_offset), _return1, _return2, _return3));
    }
  }
}

int Calendar_Date::get_time(double timezone, int& _return1, int& _return2, int& _return3) {
  // Returns multiple values of hours, minutes, seconds, milliseconds for
  // the calendar date `date' in `timezone'.  `timezone' is the number of hours added to UTC
  // to get local time.  It is in the range -12.0 to +14.0 where UTC is zone 0.0
  { Calendar_Date* date = this;

    if (timezone == 0.0) {
      return (decode_time_in_millis(date->time_millis, _return1, _return2, _return3));
    }
    else if (timezone < 0.0) {
      return (decode_time_in_millis(((((int)(date->time_millis + MILLIS_PER_DAY + (timezone * MILLIS_PER_HOUR)))) % MILLIS_PER_DAY), _return1, _return2, _return3));
    }
    else {
      return (decode_time_in_millis(((((int)(date->time_millis + (timezone * MILLIS_PER_HOUR)))) % MILLIS_PER_DAY), _return1, _return2, _return3));
    }
  }
}

Decoded_Date_Time* Calendar_Date::decode_calendar_date(double timezone) {
  // Returns a decoded time object for `date' interpreted in `timezone'
  // `timezone' is the number of hours added to UTC to get local time.  It is 
  // in the range -12.0 to +14.0 where UTC is zone 0.0
  { Calendar_Date* date = this;

    { int hour = NULL_INTEGER;
      int min = NULL_INTEGER;
      int sec = NULL_INTEGER;
      int millis = NULL_INTEGER;

      hour = date->get_time(timezone, min, sec, millis);
      { int year = NULL_INTEGER;
        int month = NULL_INTEGER;
        int day = NULL_INTEGER;
        Keyword* dow = NULL;

        year = date->get_calendar_date(timezone, month, day, dow);
        { Decoded_Date_Time* self_000 = new_decoded_date_time();

          self_000->date_time_millis = millis;
          self_000->date_time_second = sec;
          self_000->date_time_minute = min;
          self_000->date_time_hour = hour;
          self_000->date_time_day = day;
          self_000->date_time_month = month;
          self_000->date_time_year = year;
          self_000->date_time_zone = timezone;
          self_000->date_time_dow = dow;
          { Decoded_Date_Time* value_000 = self_000;

            return (value_000);
          }
        }
      }
    }
  }
}

Calendar_Date* Decoded_Date_Time::encode_calendar_date() {
  // Returns a calendar date object for `time-structure'.
  { Decoded_Date_Time* time_structure = this;

    return (make_date_time(time_structure->date_time_year, time_structure->date_time_month, time_structure->date_time_day, time_structure->date_time_hour, time_structure->date_time_minute, time_structure->date_time_second, time_structure->date_time_millis, time_structure->date_time_zone));
  }
}

Date_Time_Object* time_add(Date_Time_Object* t1, Date_Time_Object* t2) {
  // Add `t1' to `t2'.
  // If one of `t1' or `t2' is a calendar date, then the result is a calendar date.
  // If both `t1' and `t2' are relative dates, then the result is a relative date.
  // `t1' and `t2' cannot both be calendar dates.
  { Surrogate* test_value_000 = safe_primary_type(t1);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Date_Time_Object* t1_000 = t1;
        Calendar_Date* t1 = ((Calendar_Date*)(t1_000));

        { Surrogate* test_value_001 = safe_primary_type(t2);

          if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_000 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

              { Output_String_Stream* stream_000 = new_output_string_stream();

                *(stream_000->native_stream) << "You can't add two calendar dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_000->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_001 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_001));

              return (make_calendar_date(t1->modified_julian_day + t2->days, t1->time_millis + t2->millis));
            }
          }
          else {
            { Output_String_Stream* stream_001 = new_output_string_stream();

              *(stream_001->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_001->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Date_Time_Object* t1_001 = t1;
        Relative_Date* t1 = ((Relative_Date*)(t1_001));

        { Surrogate* test_value_002 = safe_primary_type(t2);

          if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_002 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_002));

              return (make_calendar_date(t1->days + t2->modified_julian_day, t1->millis + t2->time_millis));
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_003 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_003));

              return (make_relative_date(t1->days + t2->days, t1->millis + t2->millis));
            }
          }
          else {
            { Output_String_Stream* stream_002 = new_output_string_stream();

              *(stream_002->native_stream) << "`" << test_value_002 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_002->the_string_reader());
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_003 = new_output_string_stream();

        *(stream_003->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_003->the_string_reader());
      }
    }
  }
}

Date_Time_Object* time_subtract(Date_Time_Object* t1, Date_Time_Object* t2) {
  // Subtract `t2' from `t1'.  If `t1' is a calendar date, then `t2' can
  // be either a calendar date (in which case the return value is a relative date) or it
  // can be a relative date (in which case the return value is a calendar date).  If `t1'
  // is a relative date, then `t2' must also be a relative date and a relative date is returned.
  { Surrogate* test_value_000 = safe_primary_type(t1);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Date_Time_Object* t1_000 = t1;
        Calendar_Date* t1 = ((Calendar_Date*)(t1_000));

        { Surrogate* test_value_001 = safe_primary_type(t2);

          if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_000 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

              return (make_relative_date(t1->modified_julian_day - t2->modified_julian_day, t1->time_millis - t2->time_millis));
            }
          }
          else if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_001 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_001));

              return (make_calendar_date(t1->modified_julian_day - t2->days, t1->time_millis - t2->millis));
            }
          }
          else {
            { Output_String_Stream* stream_000 = new_output_string_stream();

              *(stream_000->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_000->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Date_Time_Object* t1_001 = t1;
        Relative_Date* t1 = ((Relative_Date*)(t1_001));

        { Surrogate* test_value_002 = safe_primary_type(t2);

          if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_002 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_002));

              { Output_String_Stream* stream_001 = new_output_string_stream();

                *(stream_001->native_stream) << "You can't subtract a calendar date from a relative date: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_001->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_003 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_003));

              return (make_relative_date(t1->days - t2->days, t1->millis - t2->millis));
            }
          }
          else {
            { Output_String_Stream* stream_002 = new_output_string_stream();

              *(stream_002->native_stream) << "`" << test_value_002 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_002->the_string_reader());
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_003 = new_output_string_stream();

        *(stream_003->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_003->the_string_reader());
      }
    }
  }
}

Relative_Date* time_multiply(Object* t1, Object* t2) {
  // Multiplies a relative date by a wrapped number.  One of `t1' or `t2'
  // must be a relative date and the other a wrapped number.
  { Surrogate* test_value_000 = safe_primary_type(t1);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Object* t1_000 = t1;
        Calendar_Date* t1 = ((Calendar_Date*)(t1_000));

        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "Calendar dates cannot be involved in multiplication:  " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Object* t1_001 = t1;
        Relative_Date* t1 = ((Relative_Date*)(t1_001));

        { Surrogate* test_value_001 = safe_primary_type(t2);

          if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Object* t2_000 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

              { Output_String_Stream* stream_001 = new_output_string_stream();

                *(stream_001->native_stream) << "Calendar dates cannot be involved in multiplication:  " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_001->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Object* t2_001 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_001));

              { Output_String_Stream* stream_002 = new_output_string_stream();

                *(stream_002->native_stream) << "You can't multiply two relative dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_002->the_string_reader());
              }
            }
          }
          else if (subtype_of_integerP(test_value_001)) {
            { Object* t2_002 = t2;
              Integer_Wrapper* t2 = ((Integer_Wrapper*)(t2_002));

              return (make_relative_date(t1->days * t2->wrapper_value, t1->millis * t2->wrapper_value));
            }
          }
          else if (subtype_of_floatP(test_value_001)) {
            { Object* t2_003 = t2;
              Float_Wrapper* t2 = ((Float_Wrapper*)(t2_003));

              return (make_relative_date(round(t1->days * t2->wrapper_value), round(t1->millis * t2->wrapper_value)));
            }
          }
          else {
            { Output_String_Stream* stream_003 = new_output_string_stream();

              *(stream_003->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_003->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_of_integerP(test_value_000)) {
      { Object* t1_002 = t1;
        Integer_Wrapper* t1 = ((Integer_Wrapper*)(t1_002));

        { Surrogate* test_value_002 = safe_primary_type(t2);

          if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Object* t2_004 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_004));

              { Output_String_Stream* stream_004 = new_output_string_stream();

                *(stream_004->native_stream) << "Calendar dates cannot be involved in multiplication:  " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_004->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Object* t2_005 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_005));

              return (make_relative_date(t1->wrapper_value * t2->days, t1->wrapper_value * t2->millis));
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_NUMBER_WRAPPER)) {
            { Object* t2_006 = t2;
              Number_Wrapper* t2 = ((Number_Wrapper*)(t2_006));

              { Output_String_Stream* stream_005 = new_output_string_stream();

                *(stream_005->native_stream) << "One argument must be a relative date:  " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_005->the_string_reader());
              }
            }
          }
          else {
            { Output_String_Stream* stream_006 = new_output_string_stream();

              *(stream_006->native_stream) << "`" << test_value_002 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_006->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_of_floatP(test_value_000)) {
      { Object* t1_003 = t1;
        Float_Wrapper* t1 = ((Float_Wrapper*)(t1_003));

        { Surrogate* test_value_003 = safe_primary_type(t2);

          if (subtype_ofP(test_value_003, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Object* t2_007 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_007));

              { Output_String_Stream* stream_007 = new_output_string_stream();

                *(stream_007->native_stream) << "Calendar dates cannot be involved in multiplication:  " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_007->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_003, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Object* t2_008 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_008));

              return (make_relative_date(round(t1->wrapper_value * t2->days), round(t1->wrapper_value * t2->millis)));
            }
          }
          else if (subtype_ofP(test_value_003, SGT_DATE_TIME_STELLA_NUMBER_WRAPPER)) {
            { Object* t2_009 = t2;
              Number_Wrapper* t2 = ((Number_Wrapper*)(t2_009));

              { Output_String_Stream* stream_008 = new_output_string_stream();

                *(stream_008->native_stream) << "One argument must be a relative date:  " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_008->the_string_reader());
              }
            }
          }
          else {
            { Output_String_Stream* stream_009 = new_output_string_stream();

              *(stream_009->native_stream) << "`" << test_value_003 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_009->the_string_reader());
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_010 = new_output_string_stream();

        *(stream_010->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_010->the_string_reader());
      }
    }
  }
}

Object* time_divide(Relative_Date* t1, Object* t2) {
  // Divides the relative date `t1' by `t2'.  `t2' must be
  // either a relative date or a wrapped number.  If `t2' is a relative date,
  // then the return value will be a wrapped float.  If `t2' is a wrapped number,
  // then the reutrn value will be a relative date.
  { Surrogate* test_value_000 = safe_primary_type(t2);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Object* t2_000 = t2;
        Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "Calendar dates cannot be involved in division:  " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Object* t2_001 = t2;
        Relative_Date* t2 = ((Relative_Date*)(t2_001));

        return (wrap_float((t1->days + (((double)(t1->millis)) / MILLIS_PER_DAY)) / (t2->days + (((double)(t2->millis)) / MILLIS_PER_DAY))));
      }
    }
    else if (subtype_of_integerP(test_value_000)) {
      { Object* t2_002 = t2;
        Integer_Wrapper* t2 = ((Integer_Wrapper*)(t2_002));

        return (make_relative_date(round(((double)(t1->days)) / t2->wrapper_value), round(((double)(t1->millis)) / t2->wrapper_value)));
      }
    }
    else if (subtype_of_floatP(test_value_000)) {
      { Object* t2_003 = t2;
        Float_Wrapper* t2 = ((Float_Wrapper*)(t2_003));

        return (make_relative_date(round(t1->days / t2->wrapper_value), round(t1->millis / t2->wrapper_value)));
      }
    }
    else {
      { Output_String_Stream* stream_001 = new_output_string_stream();

        *(stream_001->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_001->the_string_reader());
      }
    }
  }
}

Object* define_time_comparison_expander(Symbol* name, Symbol* renamed_Operator) {
  return (listO(5, SYM_DATE_TIME_STELLA_DEFUN, cons(name, cons(SYM_DATE_TIME_STELLA_BOOLEAN, NIL)), listO(3, listO(3, SYM_DATE_TIME_STELLA_T1, SYM_DATE_TIME_STELLA_DATE_TIME_OBJECT, NIL), listO(3, SYM_DATE_TIME_STELLA_T2, SYM_DATE_TIME_STELLA_DATE_TIME_OBJECT, NIL), NIL), listO(5, SYM_DATE_TIME_STELLA_TYPECASE, SYM_DATE_TIME_STELLA_T1, listO(3, SYM_DATE_TIME_STELLA_CALENDAR_DATE, listO(5, SYM_DATE_TIME_STELLA_TYPECASE, SYM_DATE_TIME_STELLA_T2, listO(3, SYM_DATE_TIME_STELLA_CALENDAR_DATE, listO(5, SYM_DATE_TIME_STELLA_IF, listO(4, SYM_DATE_TIME_STELLA_e, listO(3, SYM_DATE_TIME_STELLA_MODIFIED_JULIAN_DAY, SYM_DATE_TIME_STELLA_T1, NIL), listO(3, SYM_DATE_TIME_STELLA_MODIFIED_JULIAN_DAY, SYM_DATE_TIME_STELLA_T2, NIL), NIL), listO(3, SYM_DATE_TIME_STELLA_RETURN, cons(renamed_Operator, listO(3, listO(3, SYM_DATE_TIME_STELLA_TIME_MILLIS, SYM_DATE_TIME_STELLA_T1, NIL), listO(3, SYM_DATE_TIME_STELLA_TIME_MILLIS, SYM_DATE_TIME_STELLA_T2, NIL), NIL)), NIL), listO(3, SYM_DATE_TIME_STELLA_RETURN, cons(renamed_Operator, listO(3, listO(3, SYM_DATE_TIME_STELLA_MODIFIED_JULIAN_DAY, SYM_DATE_TIME_STELLA_T1, NIL), listO(3, SYM_DATE_TIME_STELLA_MODIFIED_JULIAN_DAY, SYM_DATE_TIME_STELLA_T2, NIL), NIL)), NIL), NIL), NIL), listO(3, SYM_DATE_TIME_STELLA_RELATIVE_DATE, listO(6, SYM_DATE_TIME_STELLA_ERROR, wrap_string("Can't compare calendar and relative dates: "), SYM_DATE_TIME_STELLA_T1, wrap_string("  "), SYM_DATE_TIME_STELLA_T2, NIL), NIL), NIL), NIL), listO(3, SYM_DATE_TIME_STELLA_RELATIVE_DATE, listO(5, SYM_DATE_TIME_STELLA_TYPECASE, SYM_DATE_TIME_STELLA_T2, listO(3, SYM_DATE_TIME_STELLA_CALENDAR_DATE, listO(6, SYM_DATE_TIME_STELLA_ERROR, wrap_string("Can't compare relative and calendar dates: "), SYM_DATE_TIME_STELLA_T1, wrap_string("  "), SYM_DATE_TIME_STELLA_T2, NIL), NIL), listO(3, SYM_DATE_TIME_STELLA_RELATIVE_DATE, listO(5, SYM_DATE_TIME_STELLA_IF, listO(4, SYM_DATE_TIME_STELLA_e, listO(3, SYM_DATE_TIME_STELLA_DAYS, SYM_DATE_TIME_STELLA_T1, NIL), listO(3, SYM_DATE_TIME_STELLA_DAYS, SYM_DATE_TIME_STELLA_T2, NIL), NIL), listO(3, SYM_DATE_TIME_STELLA_RETURN, cons(renamed_Operator, listO(3, listO(3, SYM_DATE_TIME_STELLA_MILLIS, SYM_DATE_TIME_STELLA_T1, NIL), listO(3, SYM_DATE_TIME_STELLA_MILLIS, SYM_DATE_TIME_STELLA_T2, NIL), NIL)), NIL), listO(3, SYM_DATE_TIME_STELLA_RETURN, cons(renamed_Operator, listO(3, listO(3, SYM_DATE_TIME_STELLA_DAYS, SYM_DATE_TIME_STELLA_T1, NIL), listO(3, SYM_DATE_TIME_STELLA_DAYS, SYM_DATE_TIME_STELLA_T2, NIL), NIL)), NIL), NIL), NIL), NIL), NIL), NIL), NIL));
}

boolean time_eqlP(Date_Time_Object* t1, Date_Time_Object* t2) {
  { Surrogate* test_value_000 = safe_primary_type(t1);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Date_Time_Object* t1_000 = t1;
        Calendar_Date* t1 = ((Calendar_Date*)(t1_000));

        { Surrogate* test_value_001 = safe_primary_type(t2);

          if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_000 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

              if (t1->modified_julian_day == t2->modified_julian_day) {
                return (t1->time_millis == t2->time_millis);
              }
              else {
                return (t1->modified_julian_day == t2->modified_julian_day);
              }
            }
          }
          else if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_001 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_001));

              { Output_String_Stream* stream_000 = new_output_string_stream();

                *(stream_000->native_stream) << "Can't compare calendar and relative dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_000->the_string_reader());
              }
            }
          }
          else {
            { Output_String_Stream* stream_001 = new_output_string_stream();

              *(stream_001->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_001->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Date_Time_Object* t1_001 = t1;
        Relative_Date* t1 = ((Relative_Date*)(t1_001));

        { Surrogate* test_value_002 = safe_primary_type(t2);

          if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_002 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_002));

              { Output_String_Stream* stream_002 = new_output_string_stream();

                *(stream_002->native_stream) << "Can't compare relative and calendar dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_002->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_003 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_003));

              if (t1->days == t2->days) {
                return (t1->millis == t2->millis);
              }
              else {
                return (t1->days == t2->days);
              }
            }
          }
          else {
            { Output_String_Stream* stream_003 = new_output_string_stream();

              *(stream_003->native_stream) << "`" << test_value_002 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_003->the_string_reader());
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_004 = new_output_string_stream();

        *(stream_004->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_004->the_string_reader());
      }
    }
  }
}

boolean time_lessP(Date_Time_Object* t1, Date_Time_Object* t2) {
  { Surrogate* test_value_000 = safe_primary_type(t1);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Date_Time_Object* t1_000 = t1;
        Calendar_Date* t1 = ((Calendar_Date*)(t1_000));

        { Surrogate* test_value_001 = safe_primary_type(t2);

          if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_000 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

              if (t1->modified_julian_day == t2->modified_julian_day) {
                return (t1->time_millis < t2->time_millis);
              }
              else {
                return (t1->modified_julian_day < t2->modified_julian_day);
              }
            }
          }
          else if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_001 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_001));

              { Output_String_Stream* stream_000 = new_output_string_stream();

                *(stream_000->native_stream) << "Can't compare calendar and relative dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_000->the_string_reader());
              }
            }
          }
          else {
            { Output_String_Stream* stream_001 = new_output_string_stream();

              *(stream_001->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_001->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Date_Time_Object* t1_001 = t1;
        Relative_Date* t1 = ((Relative_Date*)(t1_001));

        { Surrogate* test_value_002 = safe_primary_type(t2);

          if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_002 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_002));

              { Output_String_Stream* stream_002 = new_output_string_stream();

                *(stream_002->native_stream) << "Can't compare relative and calendar dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_002->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_003 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_003));

              if (t1->days == t2->days) {
                return (t1->millis < t2->millis);
              }
              else {
                return (t1->days < t2->days);
              }
            }
          }
          else {
            { Output_String_Stream* stream_003 = new_output_string_stream();

              *(stream_003->native_stream) << "`" << test_value_002 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_003->the_string_reader());
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_004 = new_output_string_stream();

        *(stream_004->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_004->the_string_reader());
      }
    }
  }
}

boolean time_greaterP(Date_Time_Object* t1, Date_Time_Object* t2) {
  { Surrogate* test_value_000 = safe_primary_type(t1);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Date_Time_Object* t1_000 = t1;
        Calendar_Date* t1 = ((Calendar_Date*)(t1_000));

        { Surrogate* test_value_001 = safe_primary_type(t2);

          if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_000 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

              if (t1->modified_julian_day == t2->modified_julian_day) {
                return (t1->time_millis > t2->time_millis);
              }
              else {
                return (t1->modified_julian_day > t2->modified_julian_day);
              }
            }
          }
          else if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_001 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_001));

              { Output_String_Stream* stream_000 = new_output_string_stream();

                *(stream_000->native_stream) << "Can't compare calendar and relative dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_000->the_string_reader());
              }
            }
          }
          else {
            { Output_String_Stream* stream_001 = new_output_string_stream();

              *(stream_001->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_001->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Date_Time_Object* t1_001 = t1;
        Relative_Date* t1 = ((Relative_Date*)(t1_001));

        { Surrogate* test_value_002 = safe_primary_type(t2);

          if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_002 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_002));

              { Output_String_Stream* stream_002 = new_output_string_stream();

                *(stream_002->native_stream) << "Can't compare relative and calendar dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_002->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_003 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_003));

              if (t1->days == t2->days) {
                return (t1->millis > t2->millis);
              }
              else {
                return (t1->days > t2->days);
              }
            }
          }
          else {
            { Output_String_Stream* stream_003 = new_output_string_stream();

              *(stream_003->native_stream) << "`" << test_value_002 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_003->the_string_reader());
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_004 = new_output_string_stream();

        *(stream_004->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_004->the_string_reader());
      }
    }
  }
}

boolean time_less_equalP(Date_Time_Object* t1, Date_Time_Object* t2) {
  { Surrogate* test_value_000 = safe_primary_type(t1);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Date_Time_Object* t1_000 = t1;
        Calendar_Date* t1 = ((Calendar_Date*)(t1_000));

        { Surrogate* test_value_001 = safe_primary_type(t2);

          if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_000 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

              if (t1->modified_julian_day == t2->modified_julian_day) {
                return (t1->time_millis <= t2->time_millis);
              }
              else {
                return (t1->modified_julian_day <= t2->modified_julian_day);
              }
            }
          }
          else if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_001 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_001));

              { Output_String_Stream* stream_000 = new_output_string_stream();

                *(stream_000->native_stream) << "Can't compare calendar and relative dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_000->the_string_reader());
              }
            }
          }
          else {
            { Output_String_Stream* stream_001 = new_output_string_stream();

              *(stream_001->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_001->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Date_Time_Object* t1_001 = t1;
        Relative_Date* t1 = ((Relative_Date*)(t1_001));

        { Surrogate* test_value_002 = safe_primary_type(t2);

          if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_002 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_002));

              { Output_String_Stream* stream_002 = new_output_string_stream();

                *(stream_002->native_stream) << "Can't compare relative and calendar dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_002->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_003 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_003));

              if (t1->days == t2->days) {
                return (t1->millis <= t2->millis);
              }
              else {
                return (t1->days <= t2->days);
              }
            }
          }
          else {
            { Output_String_Stream* stream_003 = new_output_string_stream();

              *(stream_003->native_stream) << "`" << test_value_002 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_003->the_string_reader());
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_004 = new_output_string_stream();

        *(stream_004->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_004->the_string_reader());
      }
    }
  }
}

boolean time_greater_equalP(Date_Time_Object* t1, Date_Time_Object* t2) {
  { Surrogate* test_value_000 = safe_primary_type(t1);

    if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
      { Date_Time_Object* t1_000 = t1;
        Calendar_Date* t1 = ((Calendar_Date*)(t1_000));

        { Surrogate* test_value_001 = safe_primary_type(t2);

          if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_000 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_000));

              if (t1->modified_julian_day == t2->modified_julian_day) {
                return (t1->time_millis >= t2->time_millis);
              }
              else {
                return (t1->modified_julian_day >= t2->modified_julian_day);
              }
            }
          }
          else if (subtype_ofP(test_value_001, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_001 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_001));

              { Output_String_Stream* stream_000 = new_output_string_stream();

                *(stream_000->native_stream) << "Can't compare calendar and relative dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_000->the_string_reader());
              }
            }
          }
          else {
            { Output_String_Stream* stream_001 = new_output_string_stream();

              *(stream_001->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_001->the_string_reader());
            }
          }
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
      { Date_Time_Object* t1_001 = t1;
        Relative_Date* t1 = ((Relative_Date*)(t1_001));

        { Surrogate* test_value_002 = safe_primary_type(t2);

          if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_CALENDAR_DATE)) {
            { Date_Time_Object* t2_002 = t2;
              Calendar_Date* t2 = ((Calendar_Date*)(t2_002));

              { Output_String_Stream* stream_002 = new_output_string_stream();

                *(stream_002->native_stream) << "Can't compare relative and calendar dates: " << "`" << t1 << "'" << "  " << "`" << t2 << "'";
                throw *new_stella_exception(stream_002->the_string_reader());
              }
            }
          }
          else if (subtype_ofP(test_value_002, SGT_DATE_TIME_STELLA_RELATIVE_DATE)) {
            { Date_Time_Object* t2_003 = t2;
              Relative_Date* t2 = ((Relative_Date*)(t2_003));

              if (t1->days == t2->days) {
                return (t1->millis >= t2->millis);
              }
              else {
                return (t1->days >= t2->days);
              }
            }
          }
          else {
            { Output_String_Stream* stream_003 = new_output_string_stream();

              *(stream_003->native_stream) << "`" << test_value_002 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_003->the_string_reader());
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_004 = new_output_string_stream();

        *(stream_004->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_004->the_string_reader());
      }
    }
  }
}

void help_startup_date_time1() {
  {
    KWD_DATE_TIME_MONDAY = ((Keyword*)(intern_rigid_symbol_wrt_module("MONDAY", NULL, 2)));
    KWD_DATE_TIME_TUESDAY = ((Keyword*)(intern_rigid_symbol_wrt_module("TUESDAY", NULL, 2)));
    KWD_DATE_TIME_WEDNESDAY = ((Keyword*)(intern_rigid_symbol_wrt_module("WEDNESDAY", NULL, 2)));
    KWD_DATE_TIME_THURSDAY = ((Keyword*)(intern_rigid_symbol_wrt_module("THURSDAY", NULL, 2)));
    KWD_DATE_TIME_FRIDAY = ((Keyword*)(intern_rigid_symbol_wrt_module("FRIDAY", NULL, 2)));
    KWD_DATE_TIME_SATURDAY = ((Keyword*)(intern_rigid_symbol_wrt_module("SATURDAY", NULL, 2)));
    KWD_DATE_TIME_SUNDAY = ((Keyword*)(intern_rigid_symbol_wrt_module("SUNDAY", NULL, 2)));
    KWD_DATE_TIME_NEW_MOON = ((Keyword*)(intern_rigid_symbol_wrt_module("NEW-MOON", NULL, 2)));
    KWD_DATE_TIME_FIRST_QUARTER = ((Keyword*)(intern_rigid_symbol_wrt_module("FIRST-QUARTER", NULL, 2)));
    KWD_DATE_TIME_FULL_MOON = ((Keyword*)(intern_rigid_symbol_wrt_module("FULL-MOON", NULL, 2)));
    KWD_DATE_TIME_LAST_QUARTER = ((Keyword*)(intern_rigid_symbol_wrt_module("LAST-QUARTER", NULL, 2)));
    SGT_DATE_TIME_STELLA_DECODED_DATE_TIME = ((Surrogate*)(intern_rigid_symbol_wrt_module("DECODED-DATE-TIME", NULL, 1)));
    SYM_DATE_TIME_STELLA_DATE_TIME_MILLIS = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-MILLIS", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_SECOND = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-SECOND", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_MINUTE = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-MINUTE", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_HOUR = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-HOUR", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_DAY = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-DAY", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_MONTH = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-MONTH", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_YEAR = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-YEAR", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_ZONE = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-ZONE", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_DOW = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-DOW", NULL, 0)));
    SGT_DATE_TIME_STELLA_CALENDAR_DATE = ((Surrogate*)(intern_rigid_symbol_wrt_module("CALENDAR-DATE", NULL, 1)));
    SYM_DATE_TIME_STELLA_MODIFIED_JULIAN_DAY = ((Symbol*)(intern_rigid_symbol_wrt_module("MODIFIED-JULIAN-DAY", NULL, 0)));
    SYM_DATE_TIME_STELLA_TIME_MILLIS = ((Symbol*)(intern_rigid_symbol_wrt_module("TIME-MILLIS", NULL, 0)));
    KWD_DATE_TIME_RIGHT = ((Keyword*)(intern_rigid_symbol_wrt_module("RIGHT", NULL, 2)));
    SGT_DATE_TIME_STELLA_RELATIVE_DATE = ((Surrogate*)(intern_rigid_symbol_wrt_module("RELATIVE-DATE", NULL, 1)));
    SYM_DATE_TIME_STELLA_DAYS = ((Symbol*)(intern_rigid_symbol_wrt_module("DAYS", NULL, 0)));
    SYM_DATE_TIME_STELLA_MILLIS = ((Symbol*)(intern_rigid_symbol_wrt_module("MILLIS", NULL, 0)));
    SGT_DATE_TIME_STELLA_NUMBER_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("NUMBER-WRAPPER", NULL, 1)));
    SYM_DATE_TIME_STELLA_DEFUN = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFUN", NULL, 0)));
    SYM_DATE_TIME_STELLA_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 0)));
    SYM_DATE_TIME_STELLA_T1 = ((Symbol*)(intern_rigid_symbol_wrt_module("T1", NULL, 0)));
    SYM_DATE_TIME_STELLA_DATE_TIME_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("DATE-TIME-OBJECT", NULL, 0)));
    SYM_DATE_TIME_STELLA_T2 = ((Symbol*)(intern_rigid_symbol_wrt_module("T2", NULL, 0)));
    SYM_DATE_TIME_STELLA_TYPECASE = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPECASE", NULL, 0)));
    SYM_DATE_TIME_STELLA_CALENDAR_DATE = ((Symbol*)(intern_rigid_symbol_wrt_module("CALENDAR-DATE", NULL, 0)));
    SYM_DATE_TIME_STELLA_IF = ((Symbol*)(intern_rigid_symbol_wrt_module("IF", NULL, 0)));
    SYM_DATE_TIME_STELLA_e = ((Symbol*)(intern_rigid_symbol_wrt_module("=", NULL, 0)));
    SYM_DATE_TIME_STELLA_RETURN = ((Symbol*)(intern_rigid_symbol_wrt_module("RETURN", NULL, 0)));
    SYM_DATE_TIME_STELLA_RELATIVE_DATE = ((Symbol*)(intern_rigid_symbol_wrt_module("RELATIVE-DATE", NULL, 0)));
    SYM_DATE_TIME_STELLA_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("ERROR", NULL, 0)));
    SYM_DATE_TIME_STELLA_DEFINE_TIME_COMPARISON = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINE-TIME-COMPARISON", NULL, 0)));
    SYM_DATE_TIME_STELLA_DEFINE_TIME_COMPARISON_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINE-TIME-COMPARISON-EXPANDER", NULL, 0)));
    SYM_DATE_TIME_STELLA_STARTUP_DATE_TIME = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-DATE-TIME", NULL, 0)));
    SYM_DATE_TIME_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void startup_date_time() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_date_time1();
    }
    if (current_startup_time_phaseP(4)) {
      MILLIS_PER_DAY = 1000 * 60 * 60 * 24;
      MILLIS_PER_HOUR = 1000 * 60 * 60;
      oMONTH_ABBREVIATION_VECTORo = new_vector(13);
      { int i = NULL_INTEGER;
        Object* abbreviation = NULL;
        int iter_032 = 0;
        int upper_bound_033 = 12;
        Cons* iter_034 = get_quoted_tree("((\"NUL\" \"JAN\" \"FEB\" \"MAR\" \"APR\" \"MAY\" \"JUN\" \"JUL\" \"AUG\" \"SEP\" \"OCT\" \"NOV\" \"DEC\") \"/STELLA\")", "/STELLA");

        while ((iter_032 <= upper_bound_033) &&
            (!(iter_034 == NIL))) {
          i = iter_032;
          iter_032 = iter_032 + 1;
          abbreviation = iter_034->value;
          iter_034 = iter_034->rest;
          (oMONTH_ABBREVIATION_VECTORo->the_array)[i] = (((String_Wrapper*)(abbreviation)));
        }
      }
    }
    if (current_startup_time_phaseP(5)) {
      define_class_from_stringified_source("NATIVE-DATE-TIME", "(DEFCLASS NATIVE-DATE-TIME () :PUBLIC? TRUE :DOCUMENTATION \"A data type that is used to hold the native date and time\ninformation.   It is opaque to the Stella users.  It is provided so that\nStella `calendar-date' objects can be translated into and out of native format.\nNote that there is no way to initialize a variable holding a `native-date-time'\nexcept by such a translation.\" :CPP-NATIVE-TYPE \"time_t\" :CL-NATIVE-TYPE \"INTEGER\" :JAVA-NATIVE-TYPE \"long\")");
      { Class* renamed_Class = define_class_from_stringified_source("DECODED-DATE-TIME", "(DEFCLASS DECODED-DATE-TIME (STANDARD-OBJECT) :PUBLIC? TRUE :DOCUMENTATION \"A data structure for holding decoded time values with fields\nfor easy access of the components.\" :PUBLIC-SLOTS ((DATE-TIME-MILLIS :INITIALLY 0 :TYPE INTEGER) (DATE-TIME-SECOND :INITIALLY 0 :TYPE INTEGER) (DATE-TIME-MINUTE :INITIALLY 0 :TYPE INTEGER) (DATE-TIME-HOUR :INITIALLY 0 :TYPE INTEGER) (DATE-TIME-DAY :INITIALLY 1 :TYPE INTEGER) (DATE-TIME-MONTH :INITIALLY 1 :TYPE INTEGER) (DATE-TIME-YEAR :INITIALLY 1900 :TYPE INTEGER) (DATE-TIME-ZONE :INITIALLY 0.0 :TYPE FLOAT) (DATE-TIME-DOW :INITIALLY :SUNDAY :TYPE KEYWORD)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_decoded_date_time));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_decoded_date_time_slot_value));
      }
      define_class_from_stringified_source("DATE-TIME-OBJECT", "(DEFCLASS DATE-TIME-OBJECT (STANDARD-OBJECT) :ABSTRACT? TRUE)");
      { Class* renamed_Class = define_class_from_stringified_source("CALENDAR-DATE", "(DEFCLASS CALENDAR-DATE (DATE-TIME-OBJECT) :PUBLIC? TRUE :DOCUMENTATION \"Representation of Date and Time with the date represented\nas a modified Julian day (starting at midnight instead of noon) and the\ntime as the offset in milliseconds from midnight, UTC\" :PUBLIC-SLOTS ((MODIFIED-JULIAN-DAY :TYPE INTEGER) (TIME-MILLIS :TYPE INTEGER)) :PRINT-FORM (PRINT-CALENDAR-DATE SELF STREAM))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_calendar_date));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_calendar_date_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("RELATIVE-DATE", "(DEFCLASS RELATIVE-DATE (DATE-TIME-OBJECT) :PUBLIC? TRUE :DOCUMENTATION \"Representation of Relative Days and Time with the date represented\nas a number of days and the time in milliseconds.\" :PUBLIC-SLOTS ((DAYS :TYPE INTEGER) (MILLIS :TYPE INTEGER)) :PRINT-FORM (PRINT-RELATIVE-DATE SELF STREAM))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_relative_date));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_relative_date_slot_value));
      }
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("COMPUTE-JULIAN-DAY", "(DEFUN (COMPUTE-JULIAN-DAY INTEGER) ((YYYY INTEGER) (MM INTEGER) (DD INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Returns the Julian day that starts at noon on yyyy-mm-dd.\n`yyyy' is the year.  `mm' is the month.  `dd' is the day of month.\nNegative years are B.C.  Remember there is no year zero.\")", ((cpp_function_code)(&compute_julian_day)), NULL);
      define_function_object("JULIAN-DAY-TO-MODIFIED-JULIAN-DAY", "(DEFUN (JULIAN-DAY-TO-MODIFIED-JULIAN-DAY INTEGER) ((JULIAN-DAY INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Returns the modified Julian day during which `julian-day'starts at noon.\")", ((cpp_function_code)(&julian_day_to_modified_julian_day)), NULL);
      define_function_object("MODIFIED-JULIAN-DAY-TO-JULIAN-DAY", "(DEFUN (MODIFIED-JULIAN-DAY-TO-JULIAN-DAY INTEGER) ((MODIFIED-JULIAN-DAY INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Returns the modified Julian day during which `julian-day'starts at noon.\")", ((cpp_function_code)(&modified_julian_day_to_julian_day)), NULL);
      define_function_object("COMPUTE-DAY-OF-WEEK-JULIAN", "(DEFUN (COMPUTE-DAY-OF-WEEK-JULIAN KEYWORD) ((JULIAN-DAY INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Returns the day of the week for julian-day\")", ((cpp_function_code)(&compute_day_of_week_julian)), NULL);
      define_function_object("COMPUTE-DAY-OF-WEEK", "(DEFUN (COMPUTE-DAY-OF-WEEK KEYWORD) ((YYYY INTEGER) (MM INTEGER) (DD INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Returns the day of the week for yyyy-mm-dd.\")", ((cpp_function_code)(&compute_day_of_week)), NULL);
      define_function_object("COMPUTE-NEXT-MOON-PHASE", "(DEFUN (COMPUTE-NEXT-MOON-PHASE INTEGER FLOAT) ((N INTEGER) (PHASE KEYWORD)) :PUBLIC? TRUE :DOCUMENTATION \"Returns the Julian Day and fraction of day of the Nth occurence\nsince January 1, 1900 of moon PHASE.  PHASE is one of :NEW-MOON, :FIRST-QUARTER,\n:FULL-MOON, :LAST-QUARTER\")", ((cpp_function_code)(&compute_next_moon_phase)), NULL);
      define_function_object("COMPUTE-CALENDAR-DATE", "(DEFUN (COMPUTE-CALENDAR-DATE INTEGER INTEGER INTEGER KEYWORD) ((JULIAN-DAY INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Returns the YEAR, MONTH, DAY, DAY-OF-WEEK on which the given\n`julian-day' begins at noon.\")", ((cpp_function_code)(&compute_calendar_date)), NULL);
      define_function_object("GET-CURRENT-DATE-TIME", "(DEFUN (GET-CURRENT-DATE-TIME INTEGER INTEGER INTEGER KEYWORD INTEGER INTEGER INTEGER INTEGER) () :PUBLIC? TRUE :DOCUMENTATION \"Returns the current time in UTC as multiple values of\nyear month day day-of-week hour minute second millisecond.  Currently\nmillisecond will always be zero (even in Java where it is technically available).\")", ((cpp_function_code)(&get_current_date_time)), NULL);
      define_function_object("GET-LOCAL-TIME-ZONE", "(DEFUN (GET-LOCAL-TIME-ZONE FLOAT) () :PUBLIC? TRUE :DOCUMENTATION \"Returns the current time zone offset from UTC as a float.\")", ((cpp_function_code)(&get_local_time_zone)), NULL);
      define_function_object("SET-CURRENT-TIME", "(DEFUN SET-CURRENT-TIME ((VALUES-STRUCTURE DECODED-DATE-TIME)))", ((cpp_function_code)(&set_current_time)), NULL);
      define_function_object("SET-CURRENT-DATE", "(DEFUN SET-CURRENT-DATE ((VALUES-STRUCTURE DECODED-DATE-TIME)))", ((cpp_function_code)(&set_current_date)), NULL);
      define_function_object("SET-CURRENT-DATE-TIME", "(DEFUN SET-CURRENT-DATE-TIME ((VALUES-STRUCTURE DECODED-DATE-TIME)))", ((cpp_function_code)(&set_current_date_time)), NULL);
      define_function_object("CALENDAR-DATE-TO-STRING", "(DEFUN (CALENDAR-DATE-TO-STRING STRING) ((DATE CALENDAR-DATE) (TIMEZONE FLOAT)) :PUBLIC? TRUE :DOCUMENTATION \"Returns a string representation of `date' adjusted for `timezone'\")", ((cpp_function_code)(&calendar_date_to_string)), NULL);
      define_function_object("CALENDAR-DATE-TO-NATIVE-DATE-TIME", "(DEFUN (CALENDAR-DATE-TO-NATIVE-DATE-TIME NATIVE-DATE-TIME) ((DATE CALENDAR-DATE)))", ((cpp_function_code)(&calendar_date_to_native_date_time)), NULL);
      define_function_object("NATIVE-DATE-TIME-TO-CALENDAR-DATE", "(DEFUN (NATIVE-DATE-TIME-TO-CALENDAR-DATE CALENDAR-DATE) ((DATE NATIVE-DATE-TIME)))", ((cpp_function_code)(&native_date_time_to_calendar_date)), NULL);
      define_function_object("PRINT-CALENDAR-DATE", "(DEFUN PRINT-CALENDAR-DATE ((DATE CALENDAR-DATE) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_calendar_date)), NULL);
      define_function_object("RELATIVE-DATE-TO-STRING", "(DEFUN (RELATIVE-DATE-TO-STRING STRING) ((DATE RELATIVE-DATE)) :PUBLIC? TRUE :DOCUMENTATION \"Returns a string representation of `date'\")", ((cpp_function_code)(&relative_date_to_string)), NULL);
      define_function_object("PRINT-RELATIVE-DATE", "(DEFUN PRINT-RELATIVE-DATE ((DATE RELATIVE-DATE) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_relative_date)), NULL);
      define_function_object("MAKE-DATE-TIME", "(DEFUN (MAKE-DATE-TIME CALENDAR-DATE) ((YEAR INTEGER) (MONTH INTEGER) (DAY INTEGER) (HOUR INTEGER) (MINUTE INTEGER) (SECOND INTEGER) (MILLIS INTEGER) (TIMEZONE FLOAT)) :PUBLIC? TRUE :DOCUMENTATION \"Create a calendar date with the specified components.  `year' must be\nthe complete year (i.e., a year of 98 is 98 A.D in the 1st century).\n`timezone' is a real number in the range -12.0 to +14.0 where UTC is zone 0.0;  The number\nis the number of hours to add to UTC to arrive at local time.\")", ((cpp_function_code)(&make_date_time)), NULL);
      define_function_object("MAKE-CURRENT-DATE-TIME", "(DEFUN (MAKE-CURRENT-DATE-TIME CALENDAR-DATE) () :PUBLIC? TRUE :DOCUMENTATION \"Create a calendar date with current time and date.\")", ((cpp_function_code)(&make_current_date_time)), NULL);
      define_function_object("MAKE-CALENDAR-DATE", "(DEFUN (MAKE-CALENDAR-DATE CALENDAR-DATE) ((DAY INTEGER) (TIME INTEGER)))", ((cpp_function_code)(&make_calendar_date)), NULL);
      define_function_object("MAKE-RELATIVE-DATE", "(DEFUN (MAKE-RELATIVE-DATE RELATIVE-DATE) ((DAYS INTEGER) (MILLIS INTEGER)))", ((cpp_function_code)(&make_relative_date)), NULL);
      define_function_object("DECODE-TIME-IN-MILLIS", "(DEFUN (DECODE-TIME-IN-MILLIS INTEGER INTEGER INTEGER INTEGER) ((TIME INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Returns multiple values of hours, minutes, seconds, milliseconds for\n`time' specified in milliseconds.\")", ((cpp_function_code)(&decode_time_in_millis)), NULL);
      define_method_object("(DEFMETHOD (GET-CALENDAR-DATE INTEGER INTEGER INTEGER KEYWORD) ((DATE CALENDAR-DATE) (TIMEZONE FLOAT)) :PUBLIC? TRUE :DOCUMENTATION \"Returns multiple values of year, month, day and day of week for `date'\nin `timezone'.  `timezone' is the number of hours added to UTC to get local time.  It\nis in the range -12.0 to +14.0 where UTC is zone 0.0\")", ((cpp_method_code)(&Calendar_Date::get_calendar_date)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (GET-TIME INTEGER INTEGER INTEGER INTEGER) ((DATE CALENDAR-DATE) (TIMEZONE FLOAT)) :PUBLIC? TRUE :DOCUMENTATION \"Returns multiple values of hours, minutes, seconds, milliseconds for\nthe calendar date `date' in `timezone'.  `timezone' is the number of hours added to UTC\nto get local time.  It is in the range -12.0 to +14.0 where UTC is zone 0.0\")", ((cpp_method_code)(&Calendar_Date::get_time)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (DECODE-CALENDAR-DATE DECODED-DATE-TIME) ((DATE CALENDAR-DATE) (TIMEZONE FLOAT)) :PUBLIC? TRUE :DOCUMENTATION \"Returns a decoded time object for `date' interpreted in `timezone'\n`timezone' is the number of hours added to UTC to get local time.  It is \nin the range -12.0 to +14.0 where UTC is zone 0.0\")", ((cpp_method_code)(&Calendar_Date::decode_calendar_date)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (ENCODE-CALENDAR-DATE CALENDAR-DATE) ((TIME-STRUCTURE DECODED-DATE-TIME)) :PUBLIC? TRUE :DOCUMENTATION \"Returns a calendar date object for `time-structure'.\")", ((cpp_method_code)(&Decoded_Date_Time::encode_calendar_date)), ((cpp_method_code)(NULL)));
      define_function_object("TIME-ADD", "(DEFUN (TIME-ADD DATE-TIME-OBJECT) ((T1 DATE-TIME-OBJECT) (T2 DATE-TIME-OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Add `t1' to `t2'.\nIf one of `t1' or `t2' is a calendar date, then the result is a calendar date.\nIf both `t1' and `t2' are relative dates, then the result is a relative date.\n`t1' and `t2' cannot both be calendar dates.\")", ((cpp_function_code)(&time_add)), NULL);
      define_function_object("TIME-SUBTRACT", "(DEFUN (TIME-SUBTRACT DATE-TIME-OBJECT) ((T1 DATE-TIME-OBJECT) (T2 DATE-TIME-OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Subtract `t2' from `t1'.  If `t1' is a calendar date, then `t2' can\nbe either a calendar date (in which case the return value is a relative date) or it\ncan be a relative date (in which case the return value is a calendar date).  If `t1'\nis a relative date, then `t2' must also be a relative date and a relative date is returned.\")", ((cpp_function_code)(&time_subtract)), NULL);
      define_function_object("TIME-MULTIPLY", "(DEFUN (TIME-MULTIPLY RELATIVE-DATE) ((T1 OBJECT) (T2 OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Multiplies a relative date by a wrapped number.  One of `t1' or `t2'\nmust be a relative date and the other a wrapped number.\")", ((cpp_function_code)(&time_multiply)), NULL);
      define_function_object("TIME-DIVIDE", "(DEFUN (TIME-DIVIDE OBJECT) ((T1 RELATIVE-DATE) (T2 OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Divides the relative date `t1' by `t2'.  `t2' must be\neither a relative date or a wrapped number.  If `t2' is a relative date,\nthen the return value will be a wrapped float.  If `t2' is a wrapped number,\nthen the reutrn value will be a relative date.\")", ((cpp_function_code)(&time_divide)), NULL);
      define_function_object("DEFINE-TIME-COMPARISON-EXPANDER", "(DEFUN (DEFINE-TIME-COMPARISON-EXPANDER OBJECT) ((NAME SYMBOL) (OPERATOR SYMBOL)) :PUBLIC? FALSE)", ((cpp_function_code)(&define_time_comparison_expander)), NULL);
      define_function_object("TIME-EQL?", "(DEFUN (TIME-EQL? BOOLEAN) ((T1 DATE-TIME-OBJECT) (T2 DATE-TIME-OBJECT)))", ((cpp_function_code)(&time_eqlP)), NULL);
      define_function_object("TIME-LESS?", "(DEFUN (TIME-LESS? BOOLEAN) ((T1 DATE-TIME-OBJECT) (T2 DATE-TIME-OBJECT)))", ((cpp_function_code)(&time_lessP)), NULL);
      define_function_object("TIME-GREATER?", "(DEFUN (TIME-GREATER? BOOLEAN) ((T1 DATE-TIME-OBJECT) (T2 DATE-TIME-OBJECT)))", ((cpp_function_code)(&time_greaterP)), NULL);
      define_function_object("TIME-LESS-EQUAL?", "(DEFUN (TIME-LESS-EQUAL? BOOLEAN) ((T1 DATE-TIME-OBJECT) (T2 DATE-TIME-OBJECT)))", ((cpp_function_code)(&time_less_equalP)), NULL);
      define_function_object("TIME-GREATER-EQUAL?", "(DEFUN (TIME-GREATER-EQUAL? BOOLEAN) ((T1 DATE-TIME-OBJECT) (T2 DATE-TIME-OBJECT)))", ((cpp_function_code)(&time_greater_equalP)), NULL);
      define_function_object("STARTUP-DATE-TIME", "(DEFUN STARTUP-DATE-TIME () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_date_time)), NULL);
      { Method_Slot* function = lookup_function(SYM_DATE_TIME_STELLA_STARTUP_DATE_TIME);

        set_dynamic_slot_value(function->dynamic_slots, SYM_DATE_TIME_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Date-Time"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT MILLIS-PER-DAY INTEGER (* 1000 60 60 24) :PUBLIC? FALSE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT MILLIS-PER-HOUR INTEGER (* 1000 60 60) :PUBLIC? FALSE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *MONTH-ABBREVIATION-VECTOR* (VECTOR OF STRING-WRAPPER) NULL :DOCUMENTATION \"A vector of month abbreviations for printing dates\")");
      store_macro(SYM_DATE_TIME_STELLA_DEFINE_TIME_COMPARISON, SYM_DATE_TIME_STELLA_DEFINE_TIME_COMPARISON_EXPANDER, ((cpp_function_code)(&define_time_comparison_expander)));
    }
  }
}

Keyword* KWD_DATE_TIME_MONDAY = NULL;

Keyword* KWD_DATE_TIME_TUESDAY = NULL;

Keyword* KWD_DATE_TIME_WEDNESDAY = NULL;

Keyword* KWD_DATE_TIME_THURSDAY = NULL;

Keyword* KWD_DATE_TIME_FRIDAY = NULL;

Keyword* KWD_DATE_TIME_SATURDAY = NULL;

Keyword* KWD_DATE_TIME_SUNDAY = NULL;

Keyword* KWD_DATE_TIME_NEW_MOON = NULL;

Keyword* KWD_DATE_TIME_FIRST_QUARTER = NULL;

Keyword* KWD_DATE_TIME_FULL_MOON = NULL;

Keyword* KWD_DATE_TIME_LAST_QUARTER = NULL;

Surrogate* SGT_DATE_TIME_STELLA_DECODED_DATE_TIME = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_MILLIS = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_SECOND = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_MINUTE = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_HOUR = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_DAY = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_MONTH = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_YEAR = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_ZONE = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_DOW = NULL;

Surrogate* SGT_DATE_TIME_STELLA_CALENDAR_DATE = NULL;

Symbol* SYM_DATE_TIME_STELLA_MODIFIED_JULIAN_DAY = NULL;

Symbol* SYM_DATE_TIME_STELLA_TIME_MILLIS = NULL;

Keyword* KWD_DATE_TIME_RIGHT = NULL;

Surrogate* SGT_DATE_TIME_STELLA_RELATIVE_DATE = NULL;

Symbol* SYM_DATE_TIME_STELLA_DAYS = NULL;

Symbol* SYM_DATE_TIME_STELLA_MILLIS = NULL;

Surrogate* SGT_DATE_TIME_STELLA_NUMBER_WRAPPER = NULL;

Symbol* SYM_DATE_TIME_STELLA_DEFUN = NULL;

Symbol* SYM_DATE_TIME_STELLA_BOOLEAN = NULL;

Symbol* SYM_DATE_TIME_STELLA_T1 = NULL;

Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_OBJECT = NULL;

Symbol* SYM_DATE_TIME_STELLA_T2 = NULL;

Symbol* SYM_DATE_TIME_STELLA_TYPECASE = NULL;

Symbol* SYM_DATE_TIME_STELLA_CALENDAR_DATE = NULL;

Symbol* SYM_DATE_TIME_STELLA_IF = NULL;

Symbol* SYM_DATE_TIME_STELLA_e = NULL;

Symbol* SYM_DATE_TIME_STELLA_RETURN = NULL;

Symbol* SYM_DATE_TIME_STELLA_RELATIVE_DATE = NULL;

Symbol* SYM_DATE_TIME_STELLA_ERROR = NULL;

Symbol* SYM_DATE_TIME_STELLA_DEFINE_TIME_COMPARISON = NULL;

Symbol* SYM_DATE_TIME_STELLA_DEFINE_TIME_COMPARISON_EXPANDER = NULL;

Symbol* SYM_DATE_TIME_STELLA_STARTUP_DATE_TIME = NULL;

Symbol* SYM_DATE_TIME_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
