// iterators.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Abstract_Iterator* Abstract_Iterator::allocate_iterator() {
  // Iterator objects return themselves when asked
  // for an iterator (they occupy the same position as a collection
  // within a 'foreach' statement).
  { Abstract_Iterator* self = this;

    return (self);
  }
}

int Abstract_Iterator::length() {
  // Iterate over 'self', and count how many
  // items there are.  Bad idea if 'self' iterates over an infinite
  // collection, since in that case it will run forever.'
  { Abstract_Iterator* self = this;

    { int count = 0;

      { Abstract_Iterator* i = self;

        while (i->nextP()) {
          count = count + 1;
        }
      }
      return (count);
    }
  }
}

Object* Iterator::pop() {
  // Return the first item of the sequence represented by 'self',
  // or NULL if it is empty.  Destructively uses up the first iteration element.
  { Iterator* self = this;

    if (self->nextP()) {
      return (self->value);
    }
    else {
      return (NULL);
    }
  }
}

Iterator* Iterator::advance(int n) {
  // Return 'self' after skipping over the first
  // 'n' elements in the (remainder of the) iteration.
  { Iterator* self = this;

    while ((n > 0) &&
        self->nextP()) {
      n = n - 1;
    }
    return (self);
  }
}

boolean Iterator::emptyP() {
  // Return TRUE if the sequence represented by 'self'
  // has no elements.  Side-effect free.
  { Iterator* self = this;

    throw *new_stella_exception("Missing 'empty?' method");
  }
}

boolean Iterator::memberP(Object* value) {
  // Iterate over values of 'self', returning TRUE
  // if one of them is 'eql' to 'value.
  { Iterator* self = this;

    if ((!((boolean)(value))) ||
        value->standard_objectP()) {
      { Object* m = NULL;
        Iterator* iter_000 = self;

        while (iter_000->nextP()) {
          m = iter_000->value;
          if (m == value) {
            return (TRUE);
          }
        }
      }
    }
    else {
      { Object* m = NULL;
        Iterator* iter_001 = self;

        while (iter_001->nextP()) {
          m = iter_001->value;
          if (eqlP(m, value)) {
            return (TRUE);
          }
        }
      }
    }
    return (FALSE);
  }
}

boolean Destructive_List_Iterator::nextP() {
  { Destructive_List_Iterator* self = this;

    { Cons* cursor = self->list_iterator_cursor;

      if (cursor == NIL) {
        return (FALSE);
      }
      self->value = cursor->value;
      self->list_iterator_cursor = cursor->rest;
      return (TRUE);
    }
  }
}

boolean Destructive_List_Iterator::emptyP() {
  { Destructive_List_Iterator* self = this;

    return (self->list_iterator_cursor == NIL);
  }
}

boolean terminate_destructive_list_iteratorP(Destructive_List_Iterator* self) {
  self->the_cons_list = NIL;
  self->list_iterator_cursor = NIL;
  return (TRUE);
}

void Destructive_List_Iterator::insert(Object* value) {
  // Add 'value' to the front of the list 'self'.
  { Destructive_List_Iterator* self = this;

    self->list_iterator_cursor = self->the_cons_list = cons(value, self->the_cons_list);
  }
}

Iterator* Cons::allocate_destructive_list_iterator() {
  { Cons* self = this;

    { Destructive_List_Iterator* iterator = new_destructive_list_iterator();

      iterator->the_cons_list = self;
      iterator->list_iterator_cursor = self;
      return (iterator);
    }
  }
}

Iterator* List::allocate_destructive_list_iterator() {
  { List* self = this;

    { Iterator* iterator = self->the_cons_list->allocate_destructive_list_iterator();

      self->the_cons_list = NIL;
      self->free();
      return (iterator);
    }
  }
}

boolean All_Purpose_Iterator::nextP() {
  // Apply the stored 'next?' function to 'self'.
  { All_Purpose_Iterator* self = this;

    return (((boolean  (*) (All_Purpose_Iterator*))self->iterator_next_code)(self));
  }
}

boolean All_Purpose_Iterator::emptyP() {
  { All_Purpose_Iterator* self = this;

    if (self->iterator_empty_code == NULL) {
      throw *new_stella_exception("Missing 'empty?' method");
    }
    return (((boolean  (*) ())self->iterator_empty_code)());
  }
}

boolean All_Purpose_Iterator::terminate_all_purpose_iteratorP() {
  { All_Purpose_Iterator* self = this;

    if (((boolean)(self->iterator_nested_iterator))) {
      self->iterator_nested_iterator->free();
    }
    return (TRUE);
  }
}

boolean filtered_nested_iterator_nextP(All_Purpose_Iterator* self) {
  { Iterator* nestediterator = self->iterator_nested_iterator;
    Object* value = NULL;

    while (nestediterator->nextP()) {
      value = nestediterator->value;
      if (((boolean  (*) (Object*, All_Purpose_Iterator*))self->iterator_filter_code)(value, self)) {
        self->value = value;
        return (TRUE);
      }
    }
    return (FALSE);
  }
}

boolean iterator_cons_list_nextP(All_Purpose_Iterator* self) {
  { Cons* cursor = self->iterator_cons_list;

    if (cursor == NIL) {
      return (FALSE);
    }
    self->value = cursor->value;
    self->iterator_cons_list = cursor->rest;
    return (TRUE);
  }
}

boolean iterator_cons_list_emptyP(All_Purpose_Iterator* self) {
  return (self->iterator_cons_list == NIL);
}

boolean filtered_concatenated_iterator_nextP(All_Purpose_Iterator* self) {
  { Iterator* nestediterator = self->iterator_nested_iterator;
    cpp_function_code filtercode = self->iterator_filter_code;
    Object* value = NULL;

    for (;;) {
      while (nestediterator->nextP()) {
        value = nestediterator->value;
        if (((filtercode != NULL) ? ((boolean  (*) (Object*, All_Purpose_Iterator*))filtercode)(value, self) : TRUE)) {
          self->value = value;
          return (TRUE);
        }
      }
      nestediterator = ((Iterator*)(self->iterator_cons_list->value));
      self->iterator_cons_list = self->iterator_cons_list->rest;
      if (!((boolean)(nestediterator))) {
        break;
      }
      else {
        self->iterator_nested_iterator = nestediterator;
      }
    }
    return (FALSE);
  }
}

All_Purpose_Iterator* Iterator::concatenate(Iterator* iterator2, int otheriterators, ...) {
  // Return an iterator that first generates all values of
  // `iterator1', then those of `iterator2', and then those of all `otherIterators'.
  // The generated values can be filtered by supplying a filter function to the
  // resulting iterator.
  { Iterator* iterator1 = this;

    { Cons* iteratorlist = cons(iterator2, NIL);

      { Iterator* iterator = NULL;
        va_list iter_000;
        int iter_000_count = otheriterators;
        Cons* collect_000 = NULL;

        va_start(iter_000, otheriterators);
        while ((iter_000_count--) > 0) {
          iterator = va_arg(iter_000, Iterator*);
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(iterator, NIL);
              if (iteratorlist == NIL) {
                iteratorlist = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(iteratorlist, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(iterator, NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
        va_end(iter_000);
      }
      { All_Purpose_Iterator* self_000 = new_all_purpose_iterator();

        self_000->iterator_nested_iterator = iterator1;
        self_000->iterator_cons_list = iteratorlist;
        self_000->iterator_next_code = ((cpp_function_code)(&filtered_concatenated_iterator_nextP));
        { All_Purpose_Iterator* value_000 = self_000;

          return (value_000);
        }
      }
    }
  }
}

void Integer_Interval_Iterator::initialize_integer_interval_iterator() {
  { Integer_Interval_Iterator* self = this;

    self->interval_cursor = self->lower_bound;
    self->first_iterationP = TRUE;
  }
}

boolean Integer_Interval_Iterator::nextP() {
  { Integer_Interval_Iterator* self = this;

    if (self->first_iterationP) {
      self->first_iterationP = FALSE;
    }
    else {
      self->interval_cursor = self->interval_cursor + 1;
    }
    { int upperbound = self->upper_bound;

      self->value = self->interval_cursor;
      return ((upperbound == NULL_INTEGER) ||
          (self->interval_cursor <= upperbound));
    }
  }
}

Integer_Interval_Iterator* interval(int lowerbound, int upperbound) {
  return (new_integer_interval_iterator(lowerbound, upperbound));
}

Reverse_Integer_Interval_Iterator* Integer_Interval_Iterator::reverse() {
  { Integer_Interval_Iterator* self = this;

    return (reverse_interval(self->lower_bound, self->upper_bound));
  }
}

void initialize_reverse_integer_interval_iterator(Reverse_Integer_Interval_Iterator* self) {
  self->interval_cursor = self->upper_bound;
  self->first_iterationP = TRUE;
}

boolean Reverse_Integer_Interval_Iterator::nextP() {
  { Reverse_Integer_Interval_Iterator* self = this;

    if (self->first_iterationP) {
      self->first_iterationP = FALSE;
    }
    else {
      self->interval_cursor = self->interval_cursor - 1;
    }
    { int lowerbound = self->lower_bound;

      self->value = self->interval_cursor;
      return ((lowerbound == NULL_INTEGER) ||
          (self->interval_cursor >= lowerbound));
    }
  }
}

Reverse_Integer_Interval_Iterator* reverse_interval(int lowerbound, int upperbound) {
  // Create a reverse interval object.
  return (new_reverse_integer_interval_iterator(lowerbound, upperbound));
}

String_Iterator* string_allocate_iterator(char* self) {
  { String_Iterator* self_000 = new_string_iterator();

    self_000->the_string = self;
    { String_Iterator* iterator = self_000;

      iterator->cursor = 0;
      iterator->end = strlen(self);
      return (iterator);
    }
  }
}

boolean String_Iterator::nextP() {
  { String_Iterator* self = this;

    { int pos = self->cursor;

      if (pos < self->end) {
        self->value = (self->the_string)[pos];
        self->cursor = pos + 1;
        return (TRUE);
      }
      return (FALSE);
    }
  }
}

Iterator* allocate_all_meta_objects_iterator(int arraysize, cpp_function_code nextcode, Module* module, boolean localP) {
  { All_Purpose_Iterator* iterator = new_all_purpose_iterator();
    Cons* moduleslist = NIL;

    iterator->iterator_integer = 0;
    iterator->iterator_second_integer = arraysize;
    if (((boolean)(module))) {
      if (localP) {
        moduleslist = cons(module, moduleslist);
      }
      else {
        { Module* mod = NULL;
          Iterator* iter_000 = visible_modules(module);
          Cons* collect_000 = NULL;

          while (iter_000->nextP()) {
            mod = ((Module*)(iter_000->value));
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(mod, NIL);
                if (moduleslist == NIL) {
                  moduleslist = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(moduleslist, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(mod, NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
      }
    }
    iterator->iterator_cons_list = moduleslist;
    iterator->iterator_next_code = nextcode;
    return (iterator);
  }
}

boolean selected_meta_objectP(All_Purpose_Iterator* self, Module* objectmodule) {
  { Cons* modules = self->iterator_cons_list;

    if (modules == NIL) {
      return (TRUE);
    }
    else {
      if (modules->rest == NIL) {
        if (objectmodule == modules->value) {
          return (TRUE);
        }
        else {
          return (FALSE);
        }
      }
      else {
        return (modules->membP(objectmodule));
      }
    }
  }
}

boolean all_symbols_nextP(All_Purpose_Iterator* self) {
  { int cursor = self->iterator_integer;
    Symbol* symbol = NULL;

    while (cursor < self->iterator_second_integer) {
      symbol = ((Symbol*)((oSYMBOL_ARRAYo->the_array)[cursor]));
      if (((boolean)(symbol)) &&
          selected_meta_objectP(self, ((Module*)(symbol->home_context)))) {
        self->value = symbol;
        self->iterator_integer = cursor + 1;
        return (TRUE);
      }
      cursor = cursor + 1;
    }
    return (FALSE);
  }
}

Iterator* all_symbols(Module* module, boolean localP) {
  // Iterate over all symbols visible from 'module'.
  // If 'local?', return only symbols interned in 'module'.
  // If 'module' is null, return all symbols interned everywhere.
  return (allocate_all_meta_objects_iterator(oSYMBOL_ARRAYo->top_symbol_offset + 1, ((cpp_function_code)(&all_symbols_nextP)), module, localP));
}

boolean filter_public_slotP(Slot* self, All_Purpose_Iterator* iterator) {
  iterator = iterator;
  return (self->slot_publicP);
}

boolean filter_private_slotP(Slot* self, All_Purpose_Iterator* iterator) {
  iterator = iterator;
  return (!self->slot_publicP);
}

boolean filter_method_slotP(Slot* self, All_Purpose_Iterator* iterator) {
  iterator = iterator;
  return (method_slotP(self));
}

boolean filter_public_method_slotP(Slot* self, All_Purpose_Iterator* iterator) {
  iterator = iterator;
  return (self->slot_publicP &&
      method_slotP(self));
}

boolean filter_non_external_slotP(Slot* self, All_Purpose_Iterator* iterator) {
  iterator = iterator;
  return (!self->slot_externalP);
}

boolean filter_non_parameter_slotP(Slot* self, All_Purpose_Iterator* iterator) {
  iterator = iterator;
  return (!((Class*)(self->slot_owner->surrogate_value))->class_parameters_reader()->memberP(self->slot_name));
}

boolean filter_non_auxiliary_slotP(Slot* self, All_Purpose_Iterator* iterator) {
  iterator = iterator;
  return (!((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_ITERATORS_STELLA_SLOT_AUXILIARYp, FALSE_WRAPPER)))->wrapper_value);
}

boolean filter_slotP(Slot* self, All_Purpose_Iterator* iterator) {
  { boolean alwaysP_000 = TRUE;

    { Function_Code_Wrapper* filter = NULL;
      Cons* iter_000 = ((Cons*)(iterator->iterator_object));

      while (!(iter_000 == NIL)) {
        filter = ((Function_Code_Wrapper*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean  (*) (Slot*, All_Purpose_Iterator*))filter->wrapper_value)(self, iterator)) {
          alwaysP_000 = FALSE;
          break;
        }
      }
    }
    { boolean value_000 = alwaysP_000;

      return (value_000);
    }
  }
}

boolean all_functions_nextP(All_Purpose_Iterator* self) {
  { int cursor = self->iterator_integer;
    Symbol* symbol = NULL;
    Method_Slot* function = NULL;

    while (cursor < self->iterator_second_integer) {
      symbol = ((Symbol*)((oSYMBOL_ARRAYo->the_array)[cursor]));
      if (((boolean)(symbol)) &&
          selected_meta_objectP(self, ((Module*)(symbol->home_context)))) {
        function = lookup_function(symbol);
        if (((boolean)(function)) &&
            (selected_meta_objectP(self, function->home_module()) &&
             ((self->iterator_filter_code == NULL) ||
              ((boolean  (*) (Method_Slot*, All_Purpose_Iterator*))self->iterator_filter_code)(function, self)))) {
          self->value = function;
          self->iterator_integer = cursor + 1;
          return (TRUE);
        }
      }
      cursor = cursor + 1;
    }
    return (FALSE);
  }
}

Iterator* all_functions(Module* module, boolean localP) {
  // Iterate over all functions visible from 'module'.
  // If 'local?', return only functions bound to symbols interned in 'module'.
  // If 'module' is null, return all functions defined everywhere.
  return (allocate_all_meta_objects_iterator(oSYMBOL_ARRAYo->top_symbol_offset + 1, ((cpp_function_code)(&all_functions_nextP)), module, localP));
}

Iterator* all_public_functions(Module* module, boolean localP) {
  // Iterate over all functions visible from 'module'.
  // If 'local?', return only functions bound to symbols interned in 'module'.
  // If 'module' is null, return all functions defined everywhere.
  { All_Purpose_Iterator* iterator = ((All_Purpose_Iterator*)(all_functions(module, localP)));

    iterator->iterator_filter_code = ((cpp_function_code)(&filter_public_slotP));
    return (iterator);
  }
}

boolean all_variables_nextP(All_Purpose_Iterator* self) {
  { int cursor = self->iterator_integer;
    Symbol* symbol = NULL;
    Global_Variable* variable = NULL;

    while (cursor < self->iterator_second_integer) {
      symbol = ((Symbol*)((oSYMBOL_ARRAYo->the_array)[cursor]));
      if (((boolean)(symbol)) &&
          selected_meta_objectP(self, ((Module*)(symbol->home_context)))) {
        variable = ((Global_Variable*)(oGLOBAL_VARIABLE_LOOKUP_TABLEo->lookup(symbol)));
        if (((boolean)(variable)) &&
            (selected_meta_objectP(self, variable->home_module()) &&
             ((self->iterator_filter_code == NULL) ||
              ((boolean  (*) (Global_Variable*, All_Purpose_Iterator*))self->iterator_filter_code)(variable, self)))) {
          self->value = variable;
          self->iterator_integer = cursor + 1;
          return (TRUE);
        }
      }
      cursor = cursor + 1;
    }
    return (FALSE);
  }
}

Iterator* all_variables(Module* module, boolean localP) {
  // Iterate over all variables visible from 'module'.
  // If 'local?', return only variables bound to symbols interned in 'module'.
  // If 'module' is null, return all variables defined everywhere.
  return (allocate_all_meta_objects_iterator(oSYMBOL_ARRAYo->top_symbol_offset + 1, ((cpp_function_code)(&all_variables_nextP)), module, localP));
}

boolean all_surrogates_nextP(All_Purpose_Iterator* self) {
  { int cursor = self->iterator_integer;
    Surrogate* surrogate = NULL;

    while (cursor < self->iterator_second_integer) {
      surrogate = ((Surrogate*)((oSURROGATE_ARRAYo->the_array)[cursor]));
      if (((boolean)(surrogate)) &&
          selected_meta_objectP(self, ((Module*)(surrogate->home_context)))) {
        self->value = surrogate;
        self->iterator_integer = cursor + 1;
        return (TRUE);
      }
      cursor = cursor + 1;
    }
    return (FALSE);
  }
}

Iterator* all_surrogates(Module* module, boolean localP) {
  // Iterate over all surrogates visible from 'module'.
  // If 'local?', return only surrogates interned in 'module'.
  // If 'module' is null, return all surrogates interned everywhere.
  return (allocate_all_meta_objects_iterator(oSURROGATE_ARRAYo->top_symbol_offset + 1, ((cpp_function_code)(&all_surrogates_nextP)), module, localP));
}

boolean all_classes_nextP(All_Purpose_Iterator* self) {
  { int cursor = self->iterator_integer;
    Surrogate* surrogate = NULL;
    Object* value = NULL;

    while (cursor < self->iterator_second_integer) {
      surrogate = ((Surrogate*)((oSURROGATE_ARRAYo->the_array)[cursor]));
      if (((boolean)(surrogate)) &&
          selected_meta_objectP(self, ((Module*)(surrogate->home_context)))) {
        value = surrogate->surrogate_value;
        if (((boolean)(value)) &&
            (stella_classP(value) &&
             (((Class*)(value))->class_type == surrogate))) {
          self->value = value;
          self->iterator_integer = cursor + 1;
          return (TRUE);
        }
      }
      cursor = cursor + 1;
    }
    return (FALSE);
  }
}

Iterator* all_classes(Module* module, boolean localP) {
  // Iterate over all classes visible from 'module'.
  // If 'local?', return only classes interned in 'module'.
  // If 'module' is null, return all classes interned everywhere.
  return (allocate_all_meta_objects_iterator(oSURROGATE_ARRAYo->top_symbol_offset + 1, ((cpp_function_code)(&all_classes_nextP)), module, localP));
}

boolean slots_nextP(All_Purpose_Iterator* self) {
  { Slot* slot = NULL;

    for (;;) {
      for (;;) {
        { 
          slot = ((Slot*)(self->iterator_cons_list->value));
          if (!(((boolean)(slot)))) {
            break;
          }
        }
        self->iterator_cons_list = self->iterator_cons_list->rest;
        if ((self->iterator_filter_code == NULL) ||
            ((boolean  (*) (Slot*, All_Purpose_Iterator*))self->iterator_filter_code)(slot, self)) {
          self->value = slot;
          return (TRUE);
        }
      }
      if (!self->iterator_nested_iterator->nextP()) {
        return (FALSE);
      }
      self->iterator_cons_list = ((Class*)(self->iterator_nested_iterator->value))->class_local_slots->the_cons_list;
    }
  }
}

Iterator* all_slots(Module* module, boolean localP) {
  // Iterate over all slots visible from 'module'.
  // If 'local?', return only methods interned in 'module'.
  // If 'module' is null, return all methods interned everywhere.
  { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

    iterator->iterator_next_code = ((cpp_function_code)(&slots_nextP));
    iterator->iterator_nested_iterator = all_classes(module, localP);
    iterator->iterator_cons_list = NIL;
    return (iterator);
  }
}

Iterator* all_methods(Module* module, boolean localP) {
  // Iterate over all methods visible from 'module'.
  // If 'local?', return only methods interned in 'module'.
  // If 'module' is null, return all methods interned everywhere.
  { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

    iterator->iterator_next_code = ((cpp_function_code)(&slots_nextP));
    iterator->iterator_nested_iterator = all_classes(module, localP);
    iterator->iterator_filter_code = ((cpp_function_code)(&filter_method_slotP));
    iterator->iterator_cons_list = NIL;
    return (iterator);
  }
}

Iterator* all_public_methods(Module* module, boolean localP) {
  // Iterate over all public methods visible from 'module'.
  // If 'local?', return only methods interned in 'module'.
  // If 'module' is null, return all methods interned everywhere.
  { All_Purpose_Iterator* iterator = ((All_Purpose_Iterator*)(all_slots(module, localP)));

    iterator->iterator_filter_code = ((cpp_function_code)(&filter_public_method_slotP));
    return (iterator);
  }
}

boolean class_methods_nextP(All_Purpose_Iterator* self) {
  { Slot* slot = NULL;

    for (;;) {
      { 
        slot = ((Slot*)(self->iterator_cons_list->value));
        if (!(((boolean)(slot)))) {
          break;
        }
      }
      self->iterator_cons_list = self->iterator_cons_list->rest;
      if (method_slotP(slot) &&
          ((self->iterator_filter_code == NULL) ||
           ((boolean  (*) (Slot*, All_Purpose_Iterator*))self->iterator_filter_code)(slot, self))) {
        self->value = slot;
        return (TRUE);
      }
    }
    return (FALSE);
  }
}

Iterator* class_methods(Class* renamed_Class, Cons* filters) {
  { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

    iterator->iterator_next_code = ((cpp_function_code)(&class_methods_nextP));
    iterator->iterator_cons_list = renamed_Class->class_local_slots->the_cons_list;
    iterator->iterator_object = filters;
    iterator->iterator_filter_code = ((cpp_function_code)(&filter_slotP));
    return (iterator);
  }
}

Iterator* public_class_methods(Class* renamed_Class) {
  // Iterate over all private methods attached to 'class'.
  return (class_methods(renamed_Class, cons(wrap_function_code(((cpp_function_code)(&filter_public_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_external_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_auxiliary_slotP))), NIL)))));
}

Iterator* private_class_methods(Class* renamed_Class) {
  // Iterate over all private methods attached to 'class'.
  return (class_methods(renamed_Class, cons(wrap_function_code(((cpp_function_code)(&filter_private_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_external_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_auxiliary_slotP))), NIL)))));
}

boolean class_storage_slots_nextP(All_Purpose_Iterator* self) {
  { Slot* slot = NULL;

    for (;;) {
      { 
        slot = ((Slot*)(self->iterator_cons_list->value));
        if (!(((boolean)(slot)))) {
          break;
        }
      }
      self->iterator_cons_list = self->iterator_cons_list->rest;
      if (storage_slotP(slot) &&
          ((self->iterator_filter_code == NULL) ||
           ((boolean  (*) (Slot*, All_Purpose_Iterator*))self->iterator_filter_code)(slot, self))) {
        self->value = slot;
        return (TRUE);
      }
    }
    return (FALSE);
  }
}

Iterator* class_storage_slots(Class* renamed_Class, Cons* filters) {
  { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

    iterator->iterator_next_code = ((cpp_function_code)(&class_storage_slots_nextP));
    iterator->iterator_cons_list = renamed_Class->class_local_slots->the_cons_list;
    iterator->iterator_object = filters;
    iterator->iterator_filter_code = ((cpp_function_code)(&filter_slotP));
    return (iterator);
  }
}

boolean class_tables_nextP(All_Purpose_Iterator* self) {
  { Slot* slot = NULL;

    for (;;) {
      { 
        slot = ((Slot*)(self->iterator_cons_list->value));
        if (!(((boolean)(slot)))) {
          break;
        }
      }
      self->iterator_cons_list = self->iterator_cons_list->rest;
      if (isaP(slot, SGT_ITERATORS_STELLA_TABLE) &&
          ((self->iterator_filter_code == NULL) ||
           ((boolean  (*) (Slot*, All_Purpose_Iterator*))self->iterator_filter_code)(slot, self))) {
        self->value = slot;
        return (TRUE);
      }
    }
    return (FALSE);
  }
}

Iterator* class_tables(Class* renamed_Class, Cons* filters) {
  { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

    iterator->iterator_next_code = ((cpp_function_code)(&class_tables_nextP));
    iterator->iterator_cons_list = renamed_Class->class_local_slots->the_cons_list;
    iterator->iterator_object = filters;
    iterator->iterator_filter_code = ((cpp_function_code)(&filter_slotP));
    return (iterator);
  }
}

Iterator* public_class_storage_slots(Class* renamed_Class) {
  // Iterate over all public storage-slots attached to 'class'.
  return (class_storage_slots(renamed_Class, cons(wrap_function_code(((cpp_function_code)(&filter_public_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_parameter_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_external_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_auxiliary_slotP))), NIL))))));
}

Iterator* private_class_storage_slots(Class* renamed_Class) {
  // Iterate over all private storage-slots attached to 'class'.
  return (class_storage_slots(renamed_Class, cons(wrap_function_code(((cpp_function_code)(&filter_private_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_parameter_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_external_slotP))), cons(wrap_function_code(((cpp_function_code)(&filter_non_auxiliary_slotP))), NIL))))));
}

void help_startup_iterators1() {
  {
    define_method_object("(DEFMETHOD (ALLOCATE-ITERATOR (LIKE SELF)) ((SELF ABSTRACT-ITERATOR)) :DOCUMENTATION \"Iterator objects return themselves when asked\nfor an iterator (they occupy the same position as a collection\nwithin a 'foreach' statement).\" :PUBLIC? TRUE)", ((cpp_method_code)(&Abstract_Iterator::allocate_iterator)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF ABSTRACT-ITERATOR)) :DOCUMENTATION \"Iterate over 'self', and count how many\nitems there are.  Bad idea if 'self' iterates over an infinite\ncollection, since in that case it will run forever.'\" :PUBLIC? TRUE)", ((cpp_method_code)(&Abstract_Iterator::length)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (POP (LIKE (ANY-VALUE SELF))) ((SELF ITERATOR)) :PUBLIC? TRUE :DOCUMENTATION \"Return the first item of the sequence represented by 'self',\nor NULL if it is empty.  Destructively uses up the first iteration element.\")", ((cpp_method_code)(&Iterator::pop)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ADVANCE (LIKE SELF)) ((SELF ITERATOR) (N INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Return 'self' after skipping over the first\n'n' elements in the (remainder of the) iteration.\")", ((cpp_method_code)(&Iterator::advance)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF ITERATOR)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if the sequence represented by 'self'\nhas no elements.  Side-effect free.\")", ((cpp_method_code)(&Iterator::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF ITERATOR) (VALUE OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Iterate over values of 'self', returning TRUE\nif one of them is 'eql' to 'value.\")", ((cpp_method_code)(&Iterator::memberP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF DESTRUCTIVE-LIST-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Destructive_List_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF DESTRUCTIVE-LIST-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Destructive_List_Iterator::emptyP)), ((cpp_method_code)(NULL)));
    define_function_object("TERMINATE-DESTRUCTIVE-LIST-ITERATOR?", "(DEFUN (TERMINATE-DESTRUCTIVE-LIST-ITERATOR? BOOLEAN) ((SELF DESTRUCTIVE-LIST-ITERATOR)))", ((cpp_function_code)(&terminate_destructive_list_iteratorP)), NULL);
    define_method_object("(DEFMETHOD INSERT ((SELF DESTRUCTIVE-LIST-ITERATOR) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Add 'value' to the front of the list 'self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Destructive_List_Iterator::insert)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ALLOCATE-DESTRUCTIVE-LIST-ITERATOR (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF CONS)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::allocate_destructive_list_iterator)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ALLOCATE-DESTRUCTIVE-LIST-ITERATOR (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&List::allocate_destructive_list_iterator)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)) :DOCUMENTATION \"Apply the stored 'next?' function to 'self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&All_Purpose_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&All_Purpose_Iterator::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (TERMINATE-ALL-PURPOSE-ITERATOR? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&All_Purpose_Iterator::terminate_all_purpose_iteratorP)), ((cpp_method_code)(NULL)));
    define_function_object("FILTERED-NESTED-ITERATOR-NEXT?", "(DEFUN (FILTERED-NESTED-ITERATOR-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filtered_nested_iterator_nextP)), NULL);
    define_function_object("ITERATOR-CONS-LIST-NEXT?", "(DEFUN (ITERATOR-CONS-LIST-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&iterator_cons_list_nextP)), NULL);
    define_function_object("ITERATOR-CONS-LIST-EMPTY?", "(DEFUN (ITERATOR-CONS-LIST-EMPTY? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&iterator_cons_list_emptyP)), NULL);
    define_function_object("FILTERED-CONCATENATED-ITERATOR-NEXT?", "(DEFUN (FILTERED-CONCATENATED-ITERATOR-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filtered_concatenated_iterator_nextP)), NULL);
    define_method_object("(DEFMETHOD (CONCATENATE ALL-PURPOSE-ITERATOR) ((ITERATOR1 ITERATOR) (ITERATOR2 ITERATOR) |&REST| (OTHERITERATORS ITERATOR)) :DOCUMENTATION \"Return an iterator that first generates all values of\n`iterator1', then those of `iterator2', and then those of all `otherIterators'.\nThe generated values can be filtered by supplying a filter function to the\nresulting iterator.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Iterator::concatenate)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INITIALIZE-INTEGER-INTERVAL-ITERATOR ((SELF INTEGER-INTERVAL-ITERATOR)))", ((cpp_method_code)(&Integer_Interval_Iterator::initialize_integer_interval_iterator)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF INTEGER-INTERVAL-ITERATOR)))", ((cpp_method_code)(&Integer_Interval_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_function_object("INTERVAL", "(DEFUN (INTERVAL INTEGER-INTERVAL-ITERATOR) ((LOWERBOUND INTEGER) (UPPERBOUND INTEGER)) :PUBLIC? TRUE)", ((cpp_function_code)(&interval)), NULL);
    define_method_object("(DEFMETHOD (REVERSE REVERSE-INTEGER-INTERVAL-ITERATOR) ((SELF INTEGER-INTERVAL-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Integer_Interval_Iterator::reverse)), ((cpp_method_code)(NULL)));
    define_function_object("INITIALIZE-REVERSE-INTEGER-INTERVAL-ITERATOR", "(DEFUN INITIALIZE-REVERSE-INTEGER-INTERVAL-ITERATOR ((SELF REVERSE-INTEGER-INTERVAL-ITERATOR)))", ((cpp_function_code)(&initialize_reverse_integer_interval_iterator)), NULL);
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF REVERSE-INTEGER-INTERVAL-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Reverse_Integer_Interval_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_function_object("REVERSE-INTERVAL", "(DEFUN (REVERSE-INTERVAL REVERSE-INTEGER-INTERVAL-ITERATOR) ((LOWERBOUND INTEGER) (UPPERBOUND INTEGER)) :DOCUMENTATION \"Create a reverse interval object.\")", ((cpp_function_code)(&reverse_interval)), NULL);
    define_method_object("(DEFMETHOD (ALLOCATE-ITERATOR STRING-ITERATOR) ((SELF STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF STRING-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&String_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_function_object("ALLOCATE-ALL-META-OBJECTS-ITERATOR", "(DEFUN (ALLOCATE-ALL-META-OBJECTS-ITERATOR ITERATOR) ((ARRAYSIZE INTEGER) (NEXTCODE FUNCTION-CODE) (MODULE MODULE) (LOCAL? BOOLEAN)) :PUBLIC? TRUE)", ((cpp_function_code)(&allocate_all_meta_objects_iterator)), NULL);
    define_function_object("SELECTED-META-OBJECT?", "(DEFUN (SELECTED-META-OBJECT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR) (OBJECTMODULE MODULE)) :PUBLIC? TRUE)", ((cpp_function_code)(&selected_meta_objectP)), NULL);
    define_function_object("ALL-SYMBOLS-NEXT?", "(DEFUN (ALL-SYMBOLS-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)) :PUBLIC? TRUE)", ((cpp_function_code)(&all_symbols_nextP)), NULL);
    define_function_object("ALL-SYMBOLS", "(DEFUN (ALL-SYMBOLS (ITERATOR OF SYMBOL)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all symbols visible from 'module'.\nIf 'local?', return only symbols interned in 'module'.\nIf 'module' is null, return all symbols interned everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_symbols)), NULL);
    define_function_object("FILTER-PUBLIC-SLOT?", "(DEFUN (FILTER-PUBLIC-SLOT? BOOLEAN) ((SELF SLOT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_public_slotP)), NULL);
    define_function_object("FILTER-PRIVATE-SLOT?", "(DEFUN (FILTER-PRIVATE-SLOT? BOOLEAN) ((SELF SLOT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_private_slotP)), NULL);
    define_function_object("FILTER-METHOD-SLOT?", "(DEFUN (FILTER-METHOD-SLOT? BOOLEAN) ((SELF SLOT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_method_slotP)), NULL);
    define_function_object("FILTER-PUBLIC-METHOD-SLOT?", "(DEFUN (FILTER-PUBLIC-METHOD-SLOT? BOOLEAN) ((SELF SLOT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_public_method_slotP)), NULL);
    define_function_object("FILTER-NON-EXTERNAL-SLOT?", "(DEFUN (FILTER-NON-EXTERNAL-SLOT? BOOLEAN) ((SELF SLOT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_non_external_slotP)), NULL);
    define_function_object("FILTER-NON-PARAMETER-SLOT?", "(DEFUN (FILTER-NON-PARAMETER-SLOT? BOOLEAN) ((SELF SLOT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_non_parameter_slotP)), NULL);
    define_function_object("FILTER-NON-AUXILIARY-SLOT?", "(DEFUN (FILTER-NON-AUXILIARY-SLOT? BOOLEAN) ((SELF SLOT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_non_auxiliary_slotP)), NULL);
    define_function_object("FILTER-SLOT?", "(DEFUN (FILTER-SLOT? BOOLEAN) ((SELF SLOT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_slotP)), NULL);
    define_function_object("ALL-FUNCTIONS-NEXT?", "(DEFUN (ALL-FUNCTIONS-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&all_functions_nextP)), NULL);
    define_function_object("ALL-FUNCTIONS", "(DEFUN (ALL-FUNCTIONS (ITERATOR OF FUNCTION)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all functions visible from 'module'.\nIf 'local?', return only functions bound to symbols interned in 'module'.\nIf 'module' is null, return all functions defined everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_functions)), NULL);
    define_function_object("ALL-PUBLIC-FUNCTIONS", "(DEFUN (ALL-PUBLIC-FUNCTIONS (ITERATOR OF FUNCTION)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all functions visible from 'module'.\nIf 'local?', return only functions bound to symbols interned in 'module'.\nIf 'module' is null, return all functions defined everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_public_functions)), NULL);
    define_function_object("ALL-VARIABLES-NEXT?", "(DEFUN (ALL-VARIABLES-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&all_variables_nextP)), NULL);
    define_function_object("ALL-VARIABLES", "(DEFUN (ALL-VARIABLES (ITERATOR OF GLOBAL-VARIABLE)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all variables visible from 'module'.\nIf 'local?', return only variables bound to symbols interned in 'module'.\nIf 'module' is null, return all variables defined everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_variables)), NULL);
    define_function_object("ALL-SURROGATES-NEXT?", "(DEFUN (ALL-SURROGATES-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&all_surrogates_nextP)), NULL);
    define_function_object("ALL-SURROGATES", "(DEFUN (ALL-SURROGATES (ITERATOR OF SURROGATE)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all surrogates visible from 'module'.\nIf 'local?', return only surrogates interned in 'module'.\nIf 'module' is null, return all surrogates interned everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_surrogates)), NULL);
    define_function_object("ALL-CLASSES-NEXT?", "(DEFUN (ALL-CLASSES-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&all_classes_nextP)), NULL);
    define_function_object("ALL-CLASSES", "(DEFUN (ALL-CLASSES (ITERATOR OF CLASS)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all classes visible from 'module'.\nIf 'local?', return only classes interned in 'module'.\nIf 'module' is null, return all classes interned everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_classes)), NULL);
    define_function_object("SLOTS-NEXT?", "(DEFUN (SLOTS-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&slots_nextP)), NULL);
    define_function_object("ALL-SLOTS", "(DEFUN (ALL-SLOTS (ITERATOR OF SLOT)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all slots visible from 'module'.\nIf 'local?', return only methods interned in 'module'.\nIf 'module' is null, return all methods interned everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_slots)), NULL);
    define_function_object("ALL-METHODS", "(DEFUN (ALL-METHODS (ITERATOR OF METHOD-SLOT)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all methods visible from 'module'.\nIf 'local?', return only methods interned in 'module'.\nIf 'module' is null, return all methods interned everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_methods)), NULL);
    define_function_object("ALL-PUBLIC-METHODS", "(DEFUN (ALL-PUBLIC-METHODS (ITERATOR OF METHOD-SLOT)) ((MODULE MODULE) (LOCAL? BOOLEAN)) :DOCUMENTATION \"Iterate over all public methods visible from 'module'.\nIf 'local?', return only methods interned in 'module'.\nIf 'module' is null, return all methods interned everywhere.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_public_methods)), NULL);
    define_function_object("CLASS-METHODS-NEXT?", "(DEFUN (CLASS-METHODS-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&class_methods_nextP)), NULL);
    define_function_object("CLASS-METHODS", "(DEFUN (CLASS-METHODS (ITERATOR OF METHOD-SLOT)) ((CLASS CLASS) (FILTERS (CONS OF FUNCTION-CODE-WRAPPER))))", ((cpp_function_code)(&class_methods)), NULL);
    define_function_object("PUBLIC-CLASS-METHODS", "(DEFUN (PUBLIC-CLASS-METHODS (ITERATOR OF METHOD-SLOT)) ((CLASS CLASS)) :DOCUMENTATION \"Iterate over all private methods attached to 'class'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&public_class_methods)), NULL);
    define_function_object("PRIVATE-CLASS-METHODS", "(DEFUN (PRIVATE-CLASS-METHODS (ITERATOR OF METHOD-SLOT)) ((CLASS CLASS)) :DOCUMENTATION \"Iterate over all private methods attached to 'class'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&private_class_methods)), NULL);
    define_function_object("CLASS-STORAGE-SLOTS-NEXT?", "(DEFUN (CLASS-STORAGE-SLOTS-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&class_storage_slots_nextP)), NULL);
    define_function_object("CLASS-STORAGE-SLOTS", "(DEFUN (CLASS-STORAGE-SLOTS (ITERATOR OF STORAGE-SLOT)) ((CLASS CLASS) (FILTERS (CONS OF FUNCTION-CODE-WRAPPER))))", ((cpp_function_code)(&class_storage_slots)), NULL);
  }
}

void startup_iterators() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      SYM_ITERATORS_STELLA_SLOT_AUXILIARYp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-AUXILIARY?", NULL, 0)));
      SGT_ITERATORS_STELLA_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("TABLE", NULL, 1)));
      SYM_ITERATORS_STELLA_STARTUP_ITERATORS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-ITERATORS", NULL, 0)));
      SYM_ITERATORS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_iterators1();
      define_function_object("CLASS-TABLES-NEXT?", "(DEFUN (CLASS-TABLES-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&class_tables_nextP)), NULL);
      define_function_object("CLASS-TABLES", "(DEFUN (CLASS-TABLES (ITERATOR OF TABLE)) ((CLASS CLASS) (FILTERS (CONS OF FUNCTION-CODE-WRAPPER))))", ((cpp_function_code)(&class_tables)), NULL);
      define_function_object("PUBLIC-CLASS-STORAGE-SLOTS", "(DEFUN (PUBLIC-CLASS-STORAGE-SLOTS (ITERATOR OF STORAGE-SLOT)) ((CLASS CLASS)) :DOCUMENTATION \"Iterate over all public storage-slots attached to 'class'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&public_class_storage_slots)), NULL);
      define_function_object("PRIVATE-CLASS-STORAGE-SLOTS", "(DEFUN (PRIVATE-CLASS-STORAGE-SLOTS (ITERATOR OF STORAGE-SLOT)) ((CLASS CLASS)) :DOCUMENTATION \"Iterate over all private storage-slots attached to 'class'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&private_class_storage_slots)), NULL);
      define_function_object("STARTUP-ITERATORS", "(DEFUN STARTUP-ITERATORS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_iterators)), NULL);
      { Method_Slot* function = lookup_function(SYM_ITERATORS_STELLA_STARTUP_ITERATORS);

        set_dynamic_slot_value(function->dynamic_slots, SYM_ITERATORS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Iterators"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
  }
}

Symbol* SYM_ITERATORS_STELLA_SLOT_AUXILIARYp = NULL;

Surrogate* SGT_ITERATORS_STELLA_TABLE = NULL;

Symbol* SYM_ITERATORS_STELLA_STARTUP_ITERATORS = NULL;

Symbol* SYM_ITERATORS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
